/* automatically generated from xfwm4-workspace-dialog.glade */
#ifdef __SUNPRO_C
#pragma align 4 (workspace_dialog_ui)
#endif
#ifdef __GNUC__
static const char workspace_dialog_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char workspace_dialog_ui[] =
#endif
{
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?><interface><requires lib=\"gt"
  "k+\" version=\"3.20\"/><object class=\"GtkAdjustment\" id=\"adjustment1"
  "\"><property name=\"lower\">1</property><property name=\"upper\">100</p"
  "roperty><property name=\"step_increment\">1</property><property name=\""
  "page_increment\">10</property></object><object class=\"GtkAdjustment\" "
  "id=\"adjustment2\"><property name=\"upper\">100</property><property nam"
  "e=\"step_increment\">1</property><property name=\"page_increment\">10</"
  "property></object><object class=\"GtkAdjustment\" id=\"adjustment3\"><p"
  "roperty name=\"upper\">100</property><property name=\"step_increment\">"
  "1</property><property name=\"page_increment\">10</property></object><ob"
  "ject class=\"GtkAdjustment\" id=\"adjustment4\"><property name=\"upper\""
  ">100</property><property name=\"step_increment\">1</property><property "
  "name=\"page_increment\">10</property></object><object class=\"GtkAdjust"
  "ment\" id=\"adjustment5\"><property name=\"upper\">100</property><prope"
  "rty name=\"step_increment\">1</property><property name=\"page_increment"
  "\">10</property></object><object class=\"GtkImage\" id=\"image2\"><prop"
  "erty name=\"visible\">True</property><property name=\"can_focus\">False"
  "</property><property name=\"icon_name\">window-close-symbolic</property"
  "></object><object class=\"GtkImage\" id=\"image3\"><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">False</property><pro"
  "perty name=\"icon_name\">help-browser-symbolic</property></object><obje"
  "ct class=\"XfceTitledDialog\" id=\"main-dialog\"><property name=\"can_f"
  "ocus\">False</property><property name=\"title\" translatable=\"yes\">Wo"
  "rkspaces</property><property name=\"window_position\">center-on-parent<"
  "/property><property name=\"icon_name\">org.xfce.workspaces</property><p"
  "roperty name=\"type_hint\">dialog</property><child internal-child=\"vbo"
  "x\"><object class=\"GtkBox\" id=\"main-vbox\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property><property "
  "name=\"orientation\">vertical</property><property name=\"spacing\">2</p"
  "roperty><child><object class=\"GtkNotebook\" id=\"plug-child\"><propert"
  "y name=\"visible\">True</property><property name=\"can_focus\">True</pr"
  "operty><property name=\"border_width\">6</property><child><object class"
  "=\"GtkBox\" id=\"vbox1\"><property name=\"visible\">True</property><pro"
  "perty name=\"can_focus\">False</property><property name=\"orientation\""
  ">vertical</property><property name=\"border_width\">6</property><proper"
  "ty name=\"spacing\">6</property><child><object class=\"GtkFrame\" id=\""
  "frame4\"><property name=\"visible\">True</property><property name=\"can"
  "_focus\">False</property><property name=\"label_xalign\">0</property><p"
  "roperty name=\"shadow_type\">none</property><child><object class=\"GtkA"
  "lignment\" id=\"alignment4\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">False</property><property name=\"top_paddi"
  "ng\">6</property><property name=\"bottom_padding\">6</property><propert"
  "y name=\"left_padding\">16</property><child><object class=\"GtkBox\" id"
  "=\"hbox3\"><property name=\"visible\">True</property><property name=\"c"
  "an_focus\">False</property><property name=\"orientation\">horizontal</p"
  "roperty><property name=\"spacing\">12</property><child><object class=\""
  "GtkLabel\" id=\"label3\"><property name=\"visible\">True</property><pro"
  "perty name=\"can_focus\">False</property><property name=\"xalign\">0</p"
  "roperty><property name=\"label\" translatable=\"yes\">_Number of worksp"
  "aces:</property><property name=\"use_underline\">True</property><proper"
  "ty name=\"mnemonic_widget\">workspace_count_spinbutton</property></obje"
  "ct><packing><property name=\"expand\">False</property><property name=\""
  "fill\">True</property><property name=\"position\">0</property></packing"
  "></child><child><object class=\"GtkSpinButton\" id=\"workspace_count_sp"
  "inbutton\"><property name=\"visible\">True</property><property name=\"c"
  "an_focus\">True</property><property name=\"editable\">True</property><p"
  "roperty name=\"invisible_char\">\342\200\242</property><property name=\""
  "primary_icon_activatable\">False</property><property name=\"secondary_i"
  "con_activatable\">False</property><property name=\"primary_icon_sensiti"
  "ve\">True</property><property name=\"secondary_icon_sensitive\">True</p"
  "roperty><property name=\"adjustment\">adjustment1</property></object><p"
  "acking><property name=\"expand\">False</property><property name=\"fill\""
  ">False</property><property name=\"position\">1</property></packing></ch"
  "ild></object></child></object></child><child type=\"label\"><object cla"
  "ss=\"GtkLabel\" id=\"label7\"><property name=\"visible\">True</property"
  "><property name=\"can_focus\">False</property><property name=\"label\" "
  "translatable=\"yes\">Layout</property><property name=\"use_markup\">Tru"
  "e</property><attributes><attribute name=\"weight\" value=\"bold\"/></at"
  "tributes></object></child></object><packing><property name=\"expand\">F"
  "alse</property><property name=\"fill\">True</property><property name=\""
  "position\">0</property></packing></child><child><object class=\"GtkFram"
  "e\" id=\"frame5\"><property name=\"visible\">True</property><property n"
  "ame=\"can_focus\">False</property><property name=\"label_xalign\">0</pr"
  "operty><property name=\"shadow_type\">none</property><child><object cla"
  "ss=\"GtkAlignment\" id=\"alignment5\"><property name=\"visible\">True</"
  "property><property name=\"can_focus\">False</property><property name=\""
  "top_padding\">6</property><property name=\"left_padding\">18</property>"
  "<child><object class=\"GtkScrolledWindow\" id=\"scrolledwindow1\"><prop"
  "erty name=\"visible\">True</property><property name=\"can_focus\">True<"
  "/property><property name=\"hscrollbar_policy\">never</property><propert"
  "y name=\"vscrollbar_policy\">automatic</property><property name=\"shado"
  "w_type\">etched-in</property><child><object class=\"GtkTreeView\" id=\""
  "treeview_ws_names\"><property name=\"visible\">True</property><property"
  " name=\"can_focus\">True</property><property name=\"rules_hint\">True</"
  "property></object></child></object></child></object></child><child type"
  "=\"label\"><object class=\"GtkLabel\" id=\"label8\"><property name=\"vi"
  "sible\">True</property><property name=\"can_focus\">False</property><pr"
  "operty name=\"label\" translatable=\"yes\">Names</property><property na"
  "me=\"use_markup\">True</property><attributes><attribute name=\"weight\""
  " value=\"bold\"/></attributes></object></child></object><packing><prope"
  "rty name=\"expand\">True</property><property name=\"fill\">True</proper"
  "ty><property name=\"position\">1</property></packing></child></object><"
  "/child><child type=\"tab\"><object class=\"GtkLabel\" id=\"label2\"><pr"
  "operty name=\"visible\">True</property><property name=\"can_focus\">Fal"
  "se</property><property name=\"label\" translatable=\"yes\">_General</pr"
  "operty><property name=\"use_underline\">True</property></object><packin"
  "g><property name=\"tab_fill\">False</property></packing></child><child>"
  "<object class=\"GtkBox\" id=\"vbox2\"><property name=\"visible\">True</"
  "property><property name=\"can_focus\">False</property><property name=\""
  "orientation\">vertical</property><property name=\"border_width\">6</pro"
  "perty><property name=\"spacing\">6</property><child><object class=\"Gtk"
  "Box\" id=\"hbox1\"><property name=\"visible\">True</property><property "
  "name=\"can_focus\">False</property><property name=\"orientation\">horiz"
  "ontal</property><property name=\"spacing\">12</property><child><object "
  "class=\"GtkImage\" id=\"image1\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">False</property><property name=\"icon_"
  "name\">dialog-information-symbolic</property><property name=\"icon-size"
  "\">6</property></object><packing><property name=\"expand\">False</prope"
  "rty><property name=\"fill\">True</property><property name=\"position\">"
  "0</property></packing></child><child><object class=\"GtkLabel\" id=\"la"
  "bel5\"><property name=\"visible\">True</property><property name=\"can_f"
  "ocus\">False</property><property name=\"xalign\">0</property><property "
  "name=\"label\" translatable=\"yes\">Margins are areas on the edges of t"
  "he screen where no window will be placed</property><property name=\"wra"
  "p\">True</property></object><packing><property name=\"expand\">True</pr"
  "operty><property name=\"fill\">True</property><property name=\"position"
  "\">1</property></packing></child></object><packing><property name=\"exp"
  "and\">False</property><property name=\"fill\">True</property><property "
  "name=\"position\">0</property></packing></child><child><object class=\""
  "GtkAlignment\" id=\"alignment3\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">False</property><property name=\"xscal"
  "e\">0</property><property name=\"yscale\">0</property><child><object cl"
  "ass=\"GtkTable\" id=\"table1\"><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">False</property><property name=\"n_rows\""
  ">3</property><property name=\"n_columns\">3</property><property name=\""
  "column_spacing\">6</property><property name=\"row_spacing\">6</property"
  "><child><placeholder/></child><child><placeholder/></child><child><obje"
  "ct class=\"GtkSpinButton\" id=\"margin_left_spinbutton\"><property name"
  "=\"visible\">True</property><property name=\"can_focus\">True</property"
  "><property name=\"invisible_char\">\342\200\242</property><property nam"
  "e=\"primary_icon_activatable\">False</property><property name=\"seconda"
  "ry_icon_activatable\">False</property><property name=\"primary_icon_sen"
  "sitive\">True</property><property name=\"secondary_icon_sensitive\">Tru"
  "e</property><property name=\"adjustment\">adjustment2</property></objec"
  "t><packing><property name=\"top_attach\">1</property><property name=\"b"
  "ottom_attach\">2</property><property name=\"x_options\"/><property name"
  "=\"y_options\"/></packing></child><child><object class=\"GtkSpinButton\""
  " id=\"margin_bottom_spinbutton\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">True</property><property name=\"invisi"
  "ble_char\">\342\200\242</property><property name=\"primary_icon_activat"
  "able\">False</property><property name=\"secondary_icon_activatable\">Fa"
  "lse</property><property name=\"primary_icon_sensitive\">True</property>"
  "<property name=\"secondary_icon_sensitive\">True</property><property na"
  "me=\"adjustment\">adjustment3</property></object><packing><property nam"
  "e=\"left_attach\">1</property><property name=\"right_attach\">2</proper"
  "ty><property name=\"top_attach\">2</property><property name=\"bottom_at"
  "tach\">3</property><property name=\"x_options\"/><property name=\"y_opt"
  "ions\"/></packing></child><child><object class=\"GtkSpinButton\" id=\"m"
  "argin_right_spinbutton\"><property name=\"visible\">True</property><pro"
  "perty name=\"can_focus\">True</property><property name=\"invisible_char"
  "\">\342\200\242</property><property name=\"primary_icon_activatable\">F"
  "alse</property><property name=\"secondary_icon_activatable\">False</pro"
  "perty><property name=\"primary_icon_sensitive\">True</property><propert"
  "y name=\"secondary_icon_sensitive\">True</property><property name=\"adj"
  "ustment\">adjustment4</property></object><packing><property name=\"left"
  "_attach\">2</property><property name=\"right_attach\">3</property><prop"
  "erty name=\"top_attach\">1</property><property name=\"bottom_attach\">2"
  "</property><property name=\"x_options\"/><property name=\"y_options\"/>"
  "</packing></child><child><placeholder/></child><child><object class=\"G"
  "tkImage\" id=\"monitor_icon\"><property name=\"visible\">True</property"
  "><property name=\"can_focus\">False</property><property name=\"xpad\">6"
  "</property><property name=\"ypad\">6</property><property name=\"icon_na"
  "me\">image-missing-symbolic</property><property name=\"icon-size\">6</p"
  "roperty></object><packing><property name=\"left_attach\">1</property><p"
  "roperty name=\"right_attach\">2</property><property name=\"top_attach\""
  ">1</property><property name=\"bottom_attach\">2</property></packing></c"
  "hild><child><object class=\"GtkSpinButton\" id=\"margin_top_spinbutton\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">True</property><property name=\"invisible_char\">\342\200\242</propert"
  "y><property name=\"primary_icon_activatable\">False</property><property"
  " name=\"secondary_icon_activatable\">False</property><property name=\"p"
  "rimary_icon_sensitive\">True</property><property name=\"secondary_icon_"
  "sensitive\">True</property><property name=\"adjustment\">adjustment5</p"
  "roperty></object><packing><property name=\"left_attach\">1</property><p"
  "roperty name=\"right_attach\">2</property><property name=\"x_options\"/"
  "><property name=\"y_options\"/></packing></child><child><placeholder/><"
  "/child><child><placeholder/></child><child><placeholder/></child><child"
  "><placeholder/></child></object></child></object><packing><property nam"
  "e=\"expand\">True</property><property name=\"fill\">True</property><pro"
  "perty name=\"position\">1</property></packing></child></object><packing"
  "><property name=\"position\">1</property></packing></child><child type="
  "\"tab\"><object class=\"GtkLabel\" id=\"label6\"><property name=\"visib"
  "le\">True</property><property name=\"can_focus\">False</property><prope"
  "rty name=\"label\" translatable=\"yes\">_Margins</property><property na"
  "me=\"use_underline\">True</property></object><packing><property name=\""
  "position\">1</property><property name=\"tab_fill\">False</property></pa"
  "cking></child></object><packing><property name=\"expand\">True</propert"
  "y><property name=\"fill\">True</property><property name=\"position\">0<"
  "/property></packing></child><child internal-child=\"action_area\"><obje"
  "ct class=\"GtkButtonBox\" id=\"dialog-action_area1\"><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">False</property><p"
  "roperty name=\"orientation\">horizontal</property><property name=\"layo"
  "ut_style\">end</property><child><object class=\"GtkButton\" id=\"button"
  "2\"><property name=\"label\" translatable=\"yes\">_Help</property><prop"
  "erty name=\"use_action_appearance\">False</property><property name=\"vi"
  "sible\">True</property><property name=\"can_focus\">True</property><pro"
  "perty name=\"receives_default\">True</property><property name=\"image\""
  ">image3</property><property name=\"use_underline\">True</property></obj"
  "ect><packing><property name=\"expand\">False</property><property name=\""
  "fill\">False</property><property name=\"position\">0</property><propert"
  "y name=\"secondary\">True</property></packing></child><child><object cl"
  "ass=\"GtkButton\" id=\"button1\"><property name=\"label\" translatable="
  "\"yes\">_Close</property><property name=\"use_action_appearance\">False"
  "</property><property name=\"visible\">True</property><property name=\"c"
  "an_focus\">True</property><property name=\"receives_default\">True</pro"
  "perty><property name=\"image\">image2</property><property name=\"use_un"
  "derline\">True</property></object><packing><property name=\"expand\">Fa"
  "lse</property><property name=\"fill\">False</property><property name=\""
  "pack_type\">end</property><property name=\"position\">1</property></pac"
  "king></child></object><packing><property name=\"expand\">False</propert"
  "y><property name=\"fill\">True</property><property name=\"pack_type\">e"
  "nd</property><property name=\"position\">0</property></packing></child>"
  "</object></child><action-widgets><action-widget response=\"-11\">button"
  "2</action-widget><action-widget response=\"0\">button1</action-widget><"
  "/action-widgets></object></interface>"
};

static const unsigned workspace_dialog_ui_length = 14492u;

