/* automatically generated from xfwm4-tweaks-dialog.glade */
#ifdef __SUNPRO_C
#pragma align 4 (tweaks_dialog_ui)
#endif
#ifdef __GNUC__
static const char tweaks_dialog_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char tweaks_dialog_ui[] =
#endif
{
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?><interface><object class=\"Gt"
  "kAdjustment\" id=\"adjustment1\"><property name=\"upper\">100</property"
  "><property name=\"value\">75</property><property name=\"step_increment\""
  ">1</property><property name=\"page_increment\">10</property></object><o"
  "bject class=\"GtkAdjustment\" id=\"adjustment2\"><property name=\"upper"
  "\">100</property><property name=\"value\">90</property><property name=\""
  "step_increment\">1</property><property name=\"page_increment\">10</prop"
  "erty></object><object class=\"GtkAdjustment\" id=\"adjustment3\"><prope"
  "rty name=\"upper\">100</property><property name=\"value\">90</property>"
  "<property name=\"step_increment\">1</property><property name=\"page_inc"
  "rement\">10</property></object><object class=\"GtkAdjustment\" id=\"adj"
  "ustment4\"><property name=\"upper\">100</property><property name=\"valu"
  "e\">90</property><property name=\"step_increment\">1</property><propert"
  "y name=\"page_increment\">10</property></object><object class=\"GtkAdju"
  "stment\" id=\"adjustment5\"><property name=\"upper\">100</property><pro"
  "perty name=\"value\">90</property><property name=\"step_increment\">1</"
  "property><property name=\"page_increment\">10</property></object><objec"
  "t class=\"GtkAdjustment\" id=\"adjustment6\"><property name=\"upper\">1"
  "00</property><property name=\"value\">90</property><property name=\"ste"
  "p_increment\">1</property><property name=\"page_increment\">10</propert"
  "y></object><object class=\"GtkImage\" id=\"image1\"><property name=\"vi"
  "sible\">True</property><property name=\"can_focus\">False</property><pr"
  "operty name=\"icon_name\">window-close-symbolic</property></object><obj"
  "ect class=\"GtkImage\" id=\"image2\"><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">False</property><property name=\"i"
  "con_name\">help-browser-symbolic</property></object><object class=\"Xfc"
  "eTitledDialog\" id=\"main-dialog\"><property name=\"can_focus\">False</"
  "property><property name=\"title\" translatable=\"yes\">Window Manager T"
  "weaks</property><property name=\"window_position\">center-on-parent</pr"
  "operty><property name=\"default_height\">480</property><property name=\""
  "icon_name\">org.xfce.xfwm4-tweaks</property><property name=\"type_hint\""
  ">dialog</property><child internal-child=\"vbox\"><object class=\"GtkBox"
  "\" id=\"main-vbox\"><property name=\"visible\">True</property><property"
  " name=\"can_focus\">False</property><property name=\"orientation\">vert"
  "ical</property><child internal-child=\"action_area\"><object class=\"Gt"
  "kButtonBox\" id=\"dialog-action_area1\"><property name=\"visible\">True"
  "</property><property name=\"can_focus\">False</property><property name="
  "\"layout_style\">end</property><child><object class=\"GtkButton\" id=\""
  "button2\"><property name=\"label\" translatable=\"yes\">_Help</property"
  "><property name=\"use_action_appearance\">False</property><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">True</propert"
  "y><property name=\"receives_default\">True</property><property name=\"i"
  "mage\">image2</property><property name=\"use_underline\">True</property"
  "></object><packing><property name=\"expand\">False</property><property "
  "name=\"fill\">False</property><property name=\"position\">0</property><"
  "property name=\"secondary\">True</property></packing></child><child><ob"
  "ject class=\"GtkButton\" id=\"button1\"><property name=\"label\" transl"
  "atable=\"yes\">_Close</property><property name=\"use_action_appearance\""
  ">False</property><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">True</property><property name=\"receives_default\">Tru"
  "e</property><property name=\"image\">image1</property><property name=\""
  "use_underline\">True</property></object><packing><property name=\"expan"
  "d\">False</property><property name=\"fill\">False</property><property n"
  "ame=\"position\">1</property></packing></child></object><packing><prope"
  "rty name=\"expand\">False</property><property name=\"fill\">True</prope"
  "rty><property name=\"pack_type\">end</property><property name=\"positio"
  "n\">0</property></packing></child><child><object class=\"GtkNotebook\" "
  "id=\"plug-child\"><property name=\"visible\">True</property><property n"
  "ame=\"can_focus\">True</property><property name=\"border_width\">6</pro"
  "perty><child><object class=\"GtkBox\" id=\"vbox1\"><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">False</property><pro"
  "perty name=\"orientation\">vertical</property><property name=\"border_w"
  "idth\">12</property><property name=\"spacing\">6</property><child><obje"
  "ct class=\"GtkCheckButton\" id=\"cycle_minimum_check\"><property name=\""
  "label\" translatable=\"yes\">S_kip windows that have \"skip pager\"\nor"
  " \"skip taskbar\" properties set</property><property name=\"use_action_"
  "appearance\">False</property><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">True</property><property name=\"receives_d"
  "efault\">False</property><property name=\"use_underline\">True</propert"
  "y><property name=\"draw_indicator\">True</property></object><packing><p"
  "roperty name=\"expand\">False</property><property name=\"fill\">False</"
  "property><property name=\"position\">0</property></packing></child><chi"
  "ld><object class=\"GtkCheckButton\" id=\"cycle_minimized_check\"><prope"
  "rty name=\"label\" translatable=\"yes\">Cycle _through minimized window"
  "s in most recently used order</property><property name=\"use_action_app"
  "earance\">False</property><property name=\"visible\">True</property><pr"
  "operty name=\"can_focus\">True</property><property name=\"receives_defa"
  "ult\">False</property><property name=\"use_underline\">True</property><"
  "property name=\"draw_indicator\">True</property></object><packing><prop"
  "erty name=\"expand\">False</property><property name=\"fill\">False</pro"
  "perty><property name=\"position\">1</property></packing></child><child>"
  "<object class=\"GtkCheckButton\" id=\"cycle_hidden_check\"><property na"
  "me=\"label\" translatable=\"yes\">_Include hidden (i.e. iconified) wind"
  "ows</property><property name=\"use_action_appearance\">False</property>"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "True</property><property name=\"receives_default\">False</property><pro"
  "perty name=\"use_underline\">True</property><property name=\"draw_indic"
  "ator\">True</property></object><packing><property name=\"expand\">False"
  "</property><property name=\"fill\">False</property><property name=\"pos"
  "ition\">2</property></packing></child><child><object class=\"GtkCheckBu"
  "tton\" id=\"cycle_workspaces_check\"><property name=\"label\" translata"
  "ble=\"yes\">Cycle _through windows on all workspaces</property><propert"
  "y name=\"use_action_appearance\">False</property><property name=\"visib"
  "le\">True</property><property name=\"can_focus\">True</property><proper"
  "ty name=\"receives_default\">False</property><property name=\"use_under"
  "line\">True</property><property name=\"draw_indicator\">True</property>"
  "</object><packing><property name=\"expand\">False</property><property n"
  "ame=\"fill\">False</property><property name=\"position\">3</property></"
  "packing></child><child><object class=\"GtkCheckButton\" id=\"cycle_draw"
  "_frame\"><property name=\"label\" translatable=\"yes\">_Draw frame arou"
  "nd selected window while cycling</property><property name=\"use_action_"
  "appearance\">False</property><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">True</property><property name=\"receives_d"
  "efault\">False</property><property name=\"use_underline\">True</propert"
  "y><property name=\"draw_indicator\">True</property></object><packing><p"
  "roperty name=\"expand\">False</property><property name=\"fill\">False</"
  "property><property name=\"position\">4</property></packing></child><chi"
  "ld><object class=\"GtkCheckButton\" id=\"cycle_raise\"><property name=\""
  "label\" translatable=\"yes\">_Raise windows while cycling</property><pr"
  "operty name=\"use_action_appearance\">False</property><property name=\""
  "visible\">True</property><property name=\"can_focus\">True</property><p"
  "roperty name=\"receives_default\">False</property><property name=\"use_"
  "underline\">True</property><property name=\"draw_indicator\">True</prop"
  "erty></object><packing><property name=\"expand\">False</property><prope"
  "rty name=\"fill\">False</property><property name=\"position\">5</proper"
  "ty></packing></child><child><object class=\"GtkCheckButton\" id=\"cycle"
  "_tabwin_mode\"><property name=\"label\" translatable=\"yes\">Cycle thro"
  "ugh windows in a _list</property><property name=\"use_action_appearance"
  "\">False</property><property name=\"visible\">True</property><property "
  "name=\"can_focus\">True</property><property name=\"receives_default\">F"
  "alse</property><property name=\"use_underline\">True</property><propert"
  "y name=\"draw_indicator\">True</property></object><packing><property na"
  "me=\"expand\">False</property><property name=\"fill\">False</property><"
  "property name=\"position\">6</property></packing></child></object></chi"
  "ld><child type=\"tab\"><object class=\"GtkLabel\" id=\"label1\"><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">False</"
  "property><property name=\"label\" translatable=\"yes\">C_ycling</proper"
  "ty><property name=\"use_underline\">True</property></object><packing><p"
  "roperty name=\"tab_fill\">False</property></packing></child><child><obj"
  "ect class=\"GtkBox\" id=\"vbox3\"><property name=\"visible\">True</prop"
  "erty><property name=\"can_focus\">False</property><property name=\"orie"
  "ntation\">vertical</property><property name=\"border_width\">12</proper"
  "ty><property name=\"spacing\">6</property><child><object class=\"GtkChe"
  "ckButton\" id=\"prevent_focus_stealing_check\"><property name=\"label\""
  " translatable=\"yes\">Activate foc_us stealing prevention</property><pr"
  "operty name=\"use_action_appearance\">False</property><property name=\""
  "visible\">True</property><property name=\"can_focus\">True</property><p"
  "roperty name=\"receives_default\">False</property><property name=\"use_"
  "underline\">True</property><property name=\"draw_indicator\">True</prop"
  "erty></object><packing><property name=\"expand\">False</property><prope"
  "rty name=\"fill\">False</property><property name=\"position\">0</proper"
  "ty></packing></child><child><object class=\"GtkCheckButton\" id=\"focus"
  "_hint_check\"><property name=\"label\" translatable=\"yes\">Honor _stan"
  "dard ICCCM focus hint</property><property name=\"use_action_appearance\""
  ">False</property><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">True</property><property name=\"receives_default\">Fal"
  "se</property><property name=\"use_underline\">True</property><property "
  "name=\"draw_indicator\">True</property></object><packing><property name"
  "=\"expand\">False</property><property name=\"fill\">False</property><pr"
  "operty name=\"position\">1</property></packing></child><child><object c"
  "lass=\"GtkBox\" id=\"prevent_focus_stealing_box\"><property name=\"visi"
  "ble\">True</property><property name=\"can_focus\">False</property><prop"
  "erty name=\"orientation\">vertical</property><property name=\"spacing\""
  ">6</property><child><object class=\"GtkLabel\" id=\"label11\"><property"
  " name=\"visible\">True</property><property name=\"can_focus\">False</pr"
  "operty><property name=\"xalign\">0</property><property name=\"label\" t"
  "ranslatable=\"yes\">When a window raises itself:</property></object><pa"
  "cking><property name=\"expand\">True</property><property name=\"fill\">"
  "True</property><property name=\"position\">0</property></packing></chil"
  "d><child><object class=\"GtkAlignment\" id=\"alignment2\"><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">False</proper"
  "ty><property name=\"bottom_padding\">6</property><property name=\"left_"
  "padding\">12</property><child><object class=\"GtkBox\" id=\"vbox14\"><p"
  "roperty name=\"visible\">True</property><property name=\"can_focus\">Fa"
  "lse</property><property name=\"orientation\">vertical</property><proper"
  "ty name=\"border_width\">6</property><property name=\"spacing\">6</prop"
  "erty><child><object class=\"GtkRadioButton\" id=\"activate_action_bring"
  "_option\"><property name=\"label\" translatable=\"yes\">_Bring window o"
  "n current workspace</property><property name=\"use_action_appearance\">"
  "False</property><property name=\"visible\">True</property><property nam"
  "e=\"can_focus\">True</property><property name=\"receives_default\">Fals"
  "e</property><property name=\"use_underline\">True</property><property n"
  "ame=\"active\">True</property><property name=\"draw_indicator\">True</p"
  "roperty></object><packing><property name=\"expand\">True</property><pro"
  "perty name=\"fill\">True</property><property name=\"position\">0</prope"
  "rty></packing></child><child><object class=\"GtkRadioButton\" id=\"acti"
  "vate_action_switch_option\"><property name=\"label\" translatable=\"yes"
  "\">Switch to win_dow\'s workspace</property><property name=\"use_action"
  "_appearance\">False</property><property name=\"visible\">True</property"
  "><property name=\"can_focus\">True</property><property name=\"receives_"
  "default\">False</property><property name=\"use_underline\">True</proper"
  "ty><property name=\"active\">True</property><property name=\"draw_indic"
  "ator\">True</property><property name=\"group\">activate_action_bring_op"
  "tion</property></object><packing><property name=\"expand\">True</proper"
  "ty><property name=\"fill\">True</property><property name=\"position\">1"
  "</property></packing></child><child><object class=\"GtkRadioButton\" id"
  "=\"activate_action_none_option\"><property name=\"label\" translatable="
  "\"yes\">Do _nothing</property><property name=\"use_action_appearance\">"
  "False</property><property name=\"visible\">True</property><property nam"
  "e=\"can_focus\">True</property><property name=\"receives_default\">Fals"
  "e</property><property name=\"use_underline\">True</property><property n"
  "ame=\"active\">True</property><property name=\"draw_indicator\">True</p"
  "roperty><property name=\"group\">activate_action_bring_option</property"
  "></object><packing><property name=\"expand\">True</property><property n"
  "ame=\"fill\">True</property><property name=\"position\">2</property></p"
  "acking></child></object></child></object><packing><property name=\"expa"
  "nd\">True</property><property name=\"fill\">True</property><property na"
  "me=\"position\">1</property></packing></child></object><packing><proper"
  "ty name=\"expand\">False</property><property name=\"fill\">False</prope"
  "rty><property name=\"position\">2</property></packing></child></object>"
  "<packing><property name=\"position\">1</property></packing></child><chi"
  "ld type=\"tab\"><object class=\"GtkLabel\" id=\"label2\"><property name"
  "=\"visible\">True</property><property name=\"can_focus\">False</propert"
  "y><property name=\"label\" translatable=\"yes\">_Focus</property><prope"
  "rty name=\"use_underline\">True</property></object><packing><property n"
  "ame=\"position\">1</property><property name=\"tab_fill\">False</propert"
  "y></packing></child><child><object class=\"GtkBox\" id=\"vbox4\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">False<"
  "/property><property name=\"orientation\">vertical</property><property n"
  "ame=\"border_width\">12</property><property name=\"spacing\">6</propert"
  "y><child><object class=\"GtkBox\" id=\"hbox1\"><property name=\"visible"
  "\">True</property><property name=\"can_focus\">False</property><propert"
  "y name=\"orientation\">horizontal</property><property name=\"spacing\">"
  "12</property><child><object class=\"GtkLabel\" id=\"label7\"><property "
  "name=\"visible\">True</property><property name=\"can_focus\">False</pro"
  "perty><property name=\"xalign\">0</property><property name=\"label\" tr"
  "anslatable=\"yes\">Key used to _grab and move windows:</property><prope"
  "rty name=\"use_underline\">True</property><property name=\"mnemonic_wid"
  "get\">easy_click_combo_box</property></object><packing><property name=\""
  "expand\">False</property><property name=\"fill\">False</property><prope"
  "rty name=\"position\">0</property></packing></child><child><object clas"
  "s=\"GtkComboBox\" id=\"easy_click_combo_box\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property></object><"
  "packing><property name=\"expand\">False</property><property name=\"fill"
  "\">False</property><property name=\"position\">1</property></packing></"
  "child></object><packing><property name=\"expand\">False</property><prop"
  "erty name=\"fill\">False</property><property name=\"position\">0</prope"
  "rty></packing></child><child><object class=\"GtkCheckButton\" id=\"rais"
  "e_with_any_button_check\"><property name=\"label\" translatable=\"yes\""
  ">_Raise windows when any mouse button is pressed</property><property na"
  "me=\"use_action_appearance\">False</property><property name=\"visible\""
  ">True</property><property name=\"can_focus\">True</property><property n"
  "ame=\"receives_default\">False</property><property name=\"use_underline"
  "\">True</property><property name=\"draw_indicator\">True</property></ob"
  "ject><packing><property name=\"expand\">False</property><property name="
  "\"fill\">False</property><property name=\"position\">1</property></pack"
  "ing></child><child><object class=\"GtkCheckButton\" id=\"borderless_max"
  "imize_check\"><property name=\"label\" translatable=\"yes\">Hide frame "
  "of windows when ma_ximized</property><property name=\"use_action_appear"
  "ance\">False</property><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">True</property><property name=\"receives_default"
  "\">False</property><property name=\"use_underline\">True</property><pro"
  "perty name=\"draw_indicator\">True</property></object><packing><propert"
  "y name=\"expand\">False</property><property name=\"fill\">False</proper"
  "ty><property name=\"position\">2</property></packing></child><child><ob"
  "ject class=\"GtkCheckButton\" id=\"titleless_maximize_check\"><property"
  " name=\"label\" translatable=\"yes\">Hide title of windows when maximiz"
  "ed</property><property name=\"visible\">True</property><property name=\""
  "can_focus\">True</property><property name=\"receives_default\">False</p"
  "roperty><property name=\"use_underline\">True</property><property name="
  "\"draw_indicator\">True</property></object><packing><property name=\"ex"
  "pand\">False</property><property name=\"fill\">False</property><propert"
  "y name=\"position\">3</property></packing></child><child><object class="
  "\"GtkCheckButton\" id=\"tile_on_move_check\"><property name=\"label\" t"
  "ranslatable=\"yes\">Automatically _tile windows when moving toward the "
  "screen edge</property><property name=\"use_action_appearance\">False</p"
  "roperty><property name=\"visible\">True</property><property name=\"can_"
  "focus\">True</property><property name=\"receives_default\">False</prope"
  "rty><property name=\"use_underline\">True</property><property name=\"dr"
  "aw_indicator\">True</property></object><packing><property name=\"expand"
  "\">False</property><property name=\"fill\">False</property><property na"
  "me=\"position\">4</property></packing></child><child><object class=\"Gt"
  "kCheckButton\" id=\"snap_resist_check\"><property name=\"label\" transl"
  "atable=\"yes\">Use _edge resistance instead of window snapping</propert"
  "y><property name=\"use_action_appearance\">False</property><property na"
  "me=\"visible\">True</property><property name=\"can_focus\">True</proper"
  "ty><property name=\"receives_default\">False</property><property name=\""
  "use_underline\">True</property><property name=\"draw_indicator\">True</"
  "property></object><packing><property name=\"expand\">False</property><p"
  "roperty name=\"fill\">False</property><property name=\"position\">5</pr"
  "operty></packing></child><child><object class=\"GtkCheckButton\" id=\"m"
  "ousewheel_rollup\"><property name=\"label\" translatable=\"yes\">Use mo"
  "use wheel on title bar to ro_ll up the window</property><property name="
  "\"use_action_appearance\">False</property><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">True</property><property name"
  "=\"receives_default\">False</property><property name=\"use_underline\">"
  "True</property><property name=\"draw_indicator\">True</property></objec"
  "t><packing><property name=\"expand\">False</property><property name=\"f"
  "ill\">False</property><property name=\"position\">6</property></packing"
  "></child><child><object class=\"GtkCheckButton\" id=\"urgent_blink\"><p"
  "roperty name=\"label\" translatable=\"yes\">Notify of _urgency by makin"
  "g window\'s decoration blink</property><property name=\"use_action_appe"
  "arance\">False</property><property name=\"visible\">True</property><pro"
  "perty name=\"can_focus\">True</property><property name=\"receives_defau"
  "lt\">False</property><property name=\"use_underline\">True</property><p"
  "roperty name=\"draw_indicator\">True</property></object><packing><prope"
  "rty name=\"expand\">False</property><property name=\"fill\">False</prop"
  "erty><property name=\"position\">7</property></packing></child><child><"
  "object class=\"GtkCheckButton\" id=\"repeat_urgent_blink\"><property na"
  "me=\"label\" translatable=\"yes\">Keep urgent windows _blinking repeate"
  "dly</property><property name=\"use_action_appearance\">False</property>"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "True</property><property name=\"receives_default\">False</property><pro"
  "perty name=\"use_underline\">True</property><property name=\"draw_indic"
  "ator\">True</property></object><packing><property name=\"expand\">False"
  "</property><property name=\"fill\">False</property><property name=\"pos"
  "ition\">8</property></packing></child></object><packing><property name="
  "\"position\">2</property></packing></child><child type=\"tab\"><object "
  "class=\"GtkLabel\" id=\"label3\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">False</property><property name=\"label"
  "\" translatable=\"yes\">_Accessibility</property><property name=\"use_u"
  "nderline\">True</property></object><packing><property name=\"position\""
  ">2</property><property name=\"tab_fill\">False</property></packing></ch"
  "ild><child><object class=\"GtkBox\" id=\"vbox5\"><property name=\"visib"
  "le\">True</property><property name=\"can_focus\">False</property><prope"
  "rty name=\"orientation\">vertical</property><property name=\"border_wid"
  "th\">12</property><property name=\"spacing\">6</property><child><object"
  " class=\"GtkCheckButton\" id=\"scroll_workspaces_check\"><property name"
  "=\"label\" translatable=\"yes\">Use the _mouse wheel on the desktop to "
  "switch workspaces</property><property name=\"use_action_appearance\">Fa"
  "lse</property><property name=\"visible\">True</property><property name="
  "\"can_focus\">True</property><property name=\"receives_default\">False<"
  "/property><property name=\"use_underline\">True</property><property nam"
  "e=\"draw_indicator\">True</property></object><packing><property name=\""
  "expand\">False</property><property name=\"fill\">False</property><prope"
  "rty name=\"position\">0</property></packing></child><child><object clas"
  "s=\"GtkCheckButton\" id=\"toggle_workspaces_check\"><property name=\"la"
  "bel\" translatable=\"yes\">_Remember and recall previous workspace\nwhe"
  "n switching via keyboard shortcuts</property><property name=\"use_actio"
  "n_appearance\">False</property><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">True</property><property name=\"receives"
  "_default\">False</property><property name=\"use_underline\">True</prope"
  "rty><property name=\"draw_indicator\">True</property></object><packing>"
  "<property name=\"expand\">False</property><property name=\"fill\">False"
  "</property><property name=\"position\">1</property></packing></child><c"
  "hild><object class=\"GtkCheckButton\" id=\"wrap_layout_check\"><propert"
  "y name=\"label\" translatable=\"yes\">Wrap workspaces depending on the "
  "actual desktop _layout</property><property name=\"use_action_appearance"
  "\">False</property><property name=\"visible\">True</property><property "
  "name=\"can_focus\">True</property><property name=\"receives_default\">F"
  "alse</property><property name=\"use_underline\">True</property><propert"
  "y name=\"draw_indicator\">True</property></object><packing><property na"
  "me=\"expand\">False</property><property name=\"fill\">False</property><"
  "property name=\"position\">2</property></packing></child><child><object"
  " class=\"GtkCheckButton\" id=\"wrap_cycle_check\"><property name=\"labe"
  "l\" translatable=\"yes\">Wrap workspaces when _the first or the last wo"
  "rkspace is reached</property><property name=\"use_action_appearance\">F"
  "alse</property><property name=\"visible\">True</property><property name"
  "=\"can_focus\">True</property><property name=\"receives_default\">False"
  "</property><property name=\"use_underline\">True</property><property na"
  "me=\"draw_indicator\">True</property></object><packing><property name=\""
  "expand\">False</property><property name=\"fill\">False</property><prope"
  "rty name=\"position\">3</property></packing></child></object><packing><"
  "property name=\"position\">3</property></packing></child><child type=\""
  "tab\"><object class=\"GtkLabel\" id=\"label4\"><property name=\"visible"
  "\">True</property><property name=\"can_focus\">False</property><propert"
  "y name=\"label\" translatable=\"yes\">_Workspaces</property><property n"
  "ame=\"use_underline\">True</property></object><packing><property name=\""
  "position\">3</property><property name=\"tab_fill\">False</property></pa"
  "cking></child><child><object class=\"GtkBox\" id=\"vbox6\"><property na"
  "me=\"visible\">True</property><property name=\"can_focus\">False</prope"
  "rty><property name=\"orientation\">vertical</property><property name=\""
  "border_width\">12</property><property name=\"spacing\">6</property><chi"
  "ld><object class=\"GtkLabel\" id=\"label8\"><property name=\"visible\">"
  "True</property><property name=\"can_focus\">False</property><property n"
  "ame=\"xalign\">0</property><property name=\"label\" translatable=\"yes\""
  ">_Minimum size of windows to trigger smart placement:</property><proper"
  "ty name=\"use_underline\">True</property><property name=\"mnemonic_widg"
  "et\">placement_ratio_scale</property></object><packing><property name=\""
  "expand\">False</property><property name=\"fill\">False</property><prope"
  "rty name=\"position\">0</property></packing></child><child><object clas"
  "s=\"GtkAlignment\" id=\"alignment4\"><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">False</property><property name=\"b"
  "ottom_padding\">6</property><property name=\"left_padding\">12</propert"
  "y><child><object class=\"GtkBox\" id=\"hbox2\"><property name=\"visible"
  "\">True</property><property name=\"can_focus\">False</property><propert"
  "y name=\"orientation\">horizontal</property><property name=\"spacing\">"
  "12</property><child><object class=\"GtkLabel\" id=\"label9\"><property "
  "name=\"visible\">True</property><property name=\"can_focus\">False</pro"
  "perty><property name=\"label\" translatable=\"yes\" comments=\"Smart pl"
  "acement size\">&lt;i&gt;Small&lt;/i&gt;</property><property name=\"use_"
  "markup\">True</property></object><packing><property name=\"expand\">Fal"
  "se</property><property name=\"fill\">False</property><property name=\"p"
  "osition\">0</property></packing></child><child><object class=\"GtkScale"
  "\" id=\"placement_ratio_scale\"><property name=\"visible\">True</proper"
  "ty><property name=\"can_focus\">True</property><property name=\"orienta"
  "tion\">horizontal</property><property name=\"adjustment\">adjustment1</"
  "property><property name=\"restrict_to_fill_level\">False</property><pro"
  "perty name=\"draw_value\">False</property></object><packing><property n"
  "ame=\"expand\">True</property><property name=\"fill\">True</property><p"
  "roperty name=\"position\">1</property></packing></child><child><object "
  "class=\"GtkLabel\" id=\"label10\"><property name=\"visible\">True</prop"
  "erty><property name=\"can_focus\">False</property><property name=\"labe"
  "l\" translatable=\"yes\" comments=\"Smart placement size\">&lt;i&gt;Lar"
  "ge&lt;/i&gt;</property><property name=\"use_markup\">True</property></o"
  "bject><packing><property name=\"expand\">False</property><property name"
  "=\"fill\">True</property><property name=\"position\">2</property></pack"
  "ing></child></object></child></object><packing><property name=\"expand\""
  ">False</property><property name=\"fill\">False</property><property name"
  "=\"position\">1</property></packing></child><child><object class=\"GtkB"
  "ox\" id=\"default_placement_box\"><property name=\"visible\">True</prop"
  "erty><property name=\"can_focus\">False</property><property name=\"orie"
  "ntation\">vertical</property><property name=\"spacing\">6</property><ch"
  "ild><object class=\"GtkLabel\" id=\"label27\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property><property "
  "name=\"xalign\">0</property><property name=\"label\" translatable=\"yes"
  "\">By default, place windows:</property></object><packing><property nam"
  "e=\"expand\">True</property><property name=\"fill\">True</property><pro"
  "perty name=\"position\">0</property></packing></child><child><object cl"
  "ass=\"GtkAlignment\" id=\"alignment10\"><property name=\"visible\">True"
  "</property><property name=\"can_focus\">False</property><property name="
  "\"bottom_padding\">6</property><property name=\"left_padding\">12</prop"
  "erty><child><object class=\"GtkBox\" id=\"vbox18\"><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">False</property><pro"
  "perty name=\"orientation\">vertical</property><property name=\"border_w"
  "idth\">6</property><property name=\"spacing\">6</property><child><objec"
  "t class=\"GtkRadioButton\" id=\"placement_center_option\"><property nam"
  "e=\"label\" translatable=\"yes\">At the c_enter of the screen</property"
  "><property name=\"use_action_appearance\">False</property><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">True</propert"
  "y><property name=\"receives_default\">False</property><property name=\""
  "use_underline\">True</property><property name=\"draw_indicator\">True</"
  "property></object><packing><property name=\"expand\">True</property><pr"
  "operty name=\"fill\">True</property><property name=\"position\">0</prop"
  "erty></packing></child><child><object class=\"GtkRadioButton\" id=\"pla"
  "cement_mouse_option\"><property name=\"label\" translatable=\"yes\">U_n"
  "der the mouse pointer</property><property name=\"use_action_appearance\""
  ">False</property><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">True</property><property name=\"receives_default\">Fal"
  "se</property><property name=\"use_underline\">True</property><property "
  "name=\"active\">True</property><property name=\"draw_indicator\">True</"
  "property><property name=\"group\">placement_center_option</property></o"
  "bject><packing><property name=\"expand\">True</property><property name="
  "\"fill\">True</property><property name=\"position\">1</property></packi"
  "ng></child></object></child></object><packing><property name=\"expand\""
  ">True</property><property name=\"fill\">True</property><property name=\""
  "position\">1</property></packing></child></object><packing><property na"
  "me=\"expand\">False</property><property name=\"fill\">False</property><"
  "property name=\"position\">2</property></packing></child></object><pack"
  "ing><property name=\"position\">4</property></packing></child><child ty"
  "pe=\"tab\"><object class=\"GtkLabel\" id=\"label5\"><property name=\"vi"
  "sible\">True</property><property name=\"can_focus\">False</property><pr"
  "operty name=\"label\" translatable=\"yes\">_Placement</property><proper"
  "ty name=\"use_underline\">True</property></object><packing><property na"
  "me=\"position\">4</property><property name=\"tab_fill\">False</property"
  "></packing></child><child><object class=\"GtkBox\" id=\"vbox7\"><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">False</"
  "property><property name=\"orientation\">vertical</property><property na"
  "me=\"border_width\">12</property><property name=\"spacing\">6</property"
  "><child><object class=\"GtkCheckButton\" id=\"use_compositing_check\"><"
  "property name=\"label\" translatable=\"yes\">_Enable display compositin"
  "g</property><property name=\"use_action_appearance\">False</property><p"
  "roperty name=\"visible\">True</property><property name=\"can_focus\">Tr"
  "ue</property><property name=\"receives_default\">False</property><prope"
  "rty name=\"use_underline\">True</property><property name=\"draw_indicat"
  "or\">True</property></object><packing><property name=\"expand\">False</"
  "property><property name=\"fill\">False</property><property name=\"posit"
  "ion\">0</property></packing></child><child><object class=\"GtkScrolledW"
  "indow\" id=\"use_compositing_box\"><property name=\"visible\">True</pro"
  "perty><property name=\"sensitive\">False</property><property name=\"can"
  "_focus\">True</property><property name=\"hscrollbar_policy\">never</pro"
  "perty><property name=\"vscrollbar_policy\">automatic</property><child><"
  "object class=\"GtkViewport\" id=\"viewport1\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property><property "
  "name=\"resize_mode\">queue</property><property name=\"shadow_type\">non"
  "e</property><child><object class=\"GtkAlignment\" id=\"alignment3\"><pr"
  "operty name=\"visible\">True</property><property name=\"can_focus\">Fal"
  "se</property><property name=\"bottom_padding\">6</property><property na"
  "me=\"left_padding\">12</property><child><object class=\"GtkBox\" id=\"v"
  "box8\"><property name=\"visible\">True</property><property name=\"can_f"
  "ocus\">False</property><property name=\"orientation\">vertical</propert"
  "y><property name=\"spacing\">6</property><child><object class=\"GtkChec"
  "kButton\" id=\"unredirect_overlays_check\"><property name=\"label\" tra"
  "nslatable=\"yes\">Display _fullscreen overlay windows directly</propert"
  "y><property name=\"use_action_appearance\">False</property><property na"
  "me=\"visible\">True</property><property name=\"can_focus\">True</proper"
  "ty><property name=\"receives_default\">False</property><property name=\""
  "use_underline\">True</property><property name=\"draw_indicator\">True</"
  "property></object><packing><property name=\"expand\">False</property><p"
  "roperty name=\"fill\">True</property><property name=\"position\">0</pro"
  "perty></packing></child><child><object class=\"GtkCheckButton\" id=\"cy"
  "cle_preview_check\"><property name=\"label\" translatable=\"yes\">Show "
  "windows preview in place of icons when cycling</property><property name"
  "=\"use_action_appearance\">False</property><property name=\"visible\">T"
  "rue</property><property name=\"can_focus\">True</property><property nam"
  "e=\"receives_default\">False</property><property name=\"use_underline\""
  ">True</property><property name=\"draw_indicator\">True</property></obje"
  "ct><packing><property name=\"expand\">False</property><property name=\""
  "fill\">True</property><property name=\"position\">2</property></packing"
  "></child><child><object class=\"GtkCheckButton\" id=\"show_popup_shadow"
  "_check\"><property name=\"label\" translatable=\"yes\">Show shadows und"
  "er pop_up windows</property><property name=\"use_action_appearance\">Fa"
  "lse</property><property name=\"visible\">True</property><property name="
  "\"can_focus\">True</property><property name=\"receives_default\">False<"
  "/property><property name=\"use_underline\">True</property><property nam"
  "e=\"draw_indicator\">True</property></object><packing><property name=\""
  "expand\">False</property><property name=\"fill\">True</property><proper"
  "ty name=\"position\">3</property></packing></child><child><object class"
  "=\"GtkCheckButton\" id=\"show_dock_shadow_check\"><property name=\"labe"
  "l\" translatable=\"yes\">Show shadows under _dock windows</property><pr"
  "operty name=\"use_action_appearance\">False</property><property name=\""
  "visible\">True</property><property name=\"can_focus\">True</property><p"
  "roperty name=\"receives_default\">False</property><property name=\"use_"
  "underline\">True</property><property name=\"draw_indicator\">True</prop"
  "erty></object><packing><property name=\"expand\">False</property><prope"
  "rty name=\"fill\">True</property><property name=\"position\">4</propert"
  "y></packing></child><child><object class=\"GtkCheckButton\" id=\"show_f"
  "rame_shadow_check\"><property name=\"label\" translatable=\"yes\">Show "
  "shadows under _regular windows</property><property name=\"use_action_ap"
  "pearance\">False</property><property name=\"visible\">True</property><p"
  "roperty name=\"can_focus\">True</property><property name=\"receives_def"
  "ault\">False</property><property name=\"use_underline\">True</property>"
  "<property name=\"draw_indicator\">True</property></object><packing><pro"
  "perty name=\"expand\">False</property><property name=\"fill\">True</pro"
  "perty><property name=\"position\">5</property></packing></child><child>"
  "<object class=\"GtkCheckButton\" id=\"zoom_desktop_check\"><property na"
  "me=\"label\" translatable=\"yes\">Zoom desktop with mouse wheel</proper"
  "ty><property name=\"use_action_appearance\">False</property><property n"
  "ame=\"visible\">True</property><property name=\"can_focus\">True</prope"
  "rty><property name=\"receives_default\">False</property><property name="
  "\"use_underline\">True</property><property name=\"draw_indicator\">True"
  "</property></object><packing><property name=\"expand\">False</property>"
  "<property name=\"fill\">True</property><property name=\"position\">6</p"
  "roperty></packing></child><child><object class=\"GtkCheckButton\" id=\""
  "zoom_pointer_check\"><property name=\"label\" translatable=\"yes\">Zoom"
  " pointer along with the desktop</property><property name=\"use_action_a"
  "ppearance\">False</property><property name=\"visible\">True</property><"
  "property name=\"can_focus\">True</property><property name=\"receives_de"
  "fault\">False</property><property name=\"use_underline\">True</property"
  "><property name=\"draw_indicator\">True</property></object><packing><pr"
  "operty name=\"expand\">False</property><property name=\"fill\">True</pr"
  "operty><property name=\"position\">7</property></packing></child><child"
  "><object class=\"GtkLabel\" id=\"label12\"><property name=\"visible\">T"
  "rue</property><property name=\"can_focus\">False</property><property na"
  "me=\"xalign\">0</property><property name=\"label\" translatable=\"yes\""
  ">Opaci_ty of window decorations:</property><property name=\"use_underli"
  "ne\">True</property><property name=\"mnemonic_widget\">frame_opacity_sc"
  "ale</property></object><packing><property name=\"expand\">False</proper"
  "ty><property name=\"fill\">True</property><property name=\"position\">8"
  "</property></packing></child><child><object class=\"GtkAlignment\" id=\""
  "alignment5\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"top_padding\">2</property>"
  "<property name=\"left_padding\">12</property><child><object class=\"Gtk"
  "Box\" id=\"hbox3\"><property name=\"visible\">True</property><property "
  "name=\"can_focus\">False</property><property name=\"orientation\">horiz"
  "ontal</property><child><object class=\"GtkLabel\" id=\"label17\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">False<"
  "/property><property name=\"label\" translatable=\"yes\">&lt;i&gt;Transp"
  "arent&lt;/i&gt;</property><property name=\"use_markup\">True</property>"
  "</object><packing><property name=\"expand\">False</property><property n"
  "ame=\"fill\">False</property><property name=\"position\">0</property></"
  "packing></child><child><object class=\"GtkScale\" id=\"frame_opacity_sc"
  "ale\"><property name=\"visible\">True</property><property name=\"can_fo"
  "cus\">True</property><property name=\"orientation\">horizontal</propert"
  "y><property name=\"adjustment\">adjustment2</property><property name=\""
  "restrict_to_fill_level\">False</property><property name=\"draw_value\">"
  "False</property></object><packing><property name=\"expand\">True</prope"
  "rty><property name=\"fill\">True</property><property name=\"position\">"
  "1</property></packing></child><child><object class=\"GtkLabel\" id=\"la"
  "bel18\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"label\" translatable=\"yes\">&"
  "lt;i&gt;Opaque&lt;/i&gt;</property><property name=\"use_markup\">True</"
  "property></object><packing><property name=\"expand\">False</property><p"
  "roperty name=\"fill\">False</property><property name=\"position\">2</pr"
  "operty></packing></child></object></child></object><packing><property n"
  "ame=\"expand\">False</property><property name=\"fill\">False</property>"
  "<property name=\"position\">9</property></packing></child><child><objec"
  "t class=\"GtkLabel\" id=\"label13\"><property name=\"visible\">True</pr"
  "operty><property name=\"can_focus\">False</property><property name=\"xa"
  "lign\">0</property><property name=\"label\" translatable=\"yes\">Opacit"
  "y of _inactive windows:</property><property name=\"use_underline\">True"
  "</property><property name=\"mnemonic_widget\">inactive_opacity_scale</p"
  "roperty></object><packing><property name=\"expand\">False</property><pr"
  "operty name=\"fill\">False</property><property name=\"position\">10</pr"
  "operty></packing></child><child><object class=\"GtkAlignment\" id=\"ali"
  "gnment6\"><property name=\"visible\">True</property><property name=\"ca"
  "n_focus\">False</property><property name=\"top_padding\">2</property><p"
  "roperty name=\"left_padding\">12</property><child><object class=\"GtkBo"
  "x\" id=\"hbox4\"><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">False</property><property name=\"orientation\">horizon"
  "tal</property><child><object class=\"GtkLabel\" id=\"label19\"><propert"
  "y name=\"visible\">True</property><property name=\"can_focus\">False</p"
  "roperty><property name=\"label\" translatable=\"yes\">&lt;i&gt;Transpar"
  "ent&lt;/i&gt;</property><property name=\"use_markup\">True</property></"
  "object><packing><property name=\"expand\">False</property><property nam"
  "e=\"fill\">False</property><property name=\"position\">0</property></pa"
  "cking></child><child><object class=\"GtkScale\" id=\"inactive_opacity_s"
  "cale\"><property name=\"visible\">True</property><property name=\"can_f"
  "ocus\">True</property><property name=\"orientation\">horizontal</proper"
  "ty><property name=\"adjustment\">adjustment3</property><property name=\""
  "restrict_to_fill_level\">False</property><property name=\"draw_value\">"
  "False</property></object><packing><property name=\"expand\">True</prope"
  "rty><property name=\"fill\">True</property><property name=\"position\">"
  "1</property></packing></child><child><object class=\"GtkLabel\" id=\"la"
  "bel20\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"label\" translatable=\"yes\">&"
  "lt;i&gt;Opaque&lt;/i&gt;</property><property name=\"use_markup\">True</"
  "property></object><packing><property name=\"expand\">False</property><p"
  "roperty name=\"fill\">False</property><property name=\"position\">2</pr"
  "operty></packing></child></object></child></object><packing><property n"
  "ame=\"expand\">False</property><property name=\"fill\">False</property>"
  "<property name=\"position\">11</property></packing></child><child><obje"
  "ct class=\"GtkLabel\" id=\"label14\"><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">False</property><property name=\"x"
  "align\">0</property><property name=\"label\" translatable=\"yes\">Opaci"
  "ty of windows during _move:</property><property name=\"use_underline\">"
  "True</property><property name=\"mnemonic_widget\">move_opacity_scale</p"
  "roperty></object><packing><property name=\"expand\">False</property><pr"
  "operty name=\"fill\">False</property><property name=\"position\">12</pr"
  "operty></packing></child><child><object class=\"GtkAlignment\" id=\"ali"
  "gnment7\"><property name=\"visible\">True</property><property name=\"ca"
  "n_focus\">False</property><property name=\"top_padding\">2</property><p"
  "roperty name=\"left_padding\">12</property><child><object class=\"GtkBo"
  "x\" id=\"hbox5\"><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">False</property><property name=\"orientation\">horizon"
  "tal</property><child><object class=\"GtkLabel\" id=\"label21\"><propert"
  "y name=\"visible\">True</property><property name=\"can_focus\">False</p"
  "roperty><property name=\"label\" translatable=\"yes\">&lt;i&gt;Transpar"
  "ent&lt;/i&gt;</property><property name=\"use_markup\">True</property></"
  "object><packing><property name=\"expand\">False</property><property nam"
  "e=\"fill\">False</property><property name=\"position\">0</property></pa"
  "cking></child><child><object class=\"GtkScale\" id=\"move_opacity_scale"
  "\"><property name=\"visible\">True</property><property name=\"can_focus"
  "\">True</property><property name=\"orientation\">horizontal</property><"
  "property name=\"adjustment\">adjustment4</property><property name=\"res"
  "trict_to_fill_level\">False</property><property name=\"draw_value\">Fal"
  "se</property></object><packing><property name=\"expand\">True</property"
  "><property name=\"fill\">True</property><property name=\"position\">1</"
  "property></packing></child><child><object class=\"GtkLabel\" id=\"label"
  "22\"><property name=\"visible\">True</property><property name=\"can_foc"
  "us\">False</property><property name=\"label\" translatable=\"yes\">&lt;"
  "i&gt;Opaque&lt;/i&gt;</property><property name=\"use_markup\">True</pro"
  "perty></object><packing><property name=\"expand\">False</property><prop"
  "erty name=\"fill\">False</property><property name=\"position\">2</prope"
  "rty></packing></child></object></child></object><packing><property name"
  "=\"expand\">False</property><property name=\"fill\">False</property><pr"
  "operty name=\"position\">13</property></packing></child><child><object "
  "class=\"GtkLabel\" id=\"label15\"><property name=\"visible\">True</prop"
  "erty><property name=\"can_focus\">False</property><property name=\"xali"
  "gn\">0</property><property name=\"label\" translatable=\"yes\">Opacity "
  "of windows during resi_ze:</property><property name=\"use_underline\">T"
  "rue</property><property name=\"mnemonic_widget\">resize_opacity_scale</"
  "property></object><packing><property name=\"expand\">False</property><p"
  "roperty name=\"fill\">False</property><property name=\"position\">14</p"
  "roperty></packing></child><child><object class=\"GtkAlignment\" id=\"al"
  "ignment8\"><property name=\"visible\">True</property><property name=\"c"
  "an_focus\">False</property><property name=\"top_padding\">2</property><"
  "property name=\"left_padding\">12</property><child><object class=\"GtkB"
  "ox\" id=\"hbox6\"><property name=\"visible\">True</property><property n"
  "ame=\"can_focus\">False</property><property name=\"orientation\">horizo"
  "ntal</property><child><object class=\"GtkLabel\" id=\"label23\"><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">False</"
  "property><property name=\"label\" translatable=\"yes\">&lt;i&gt;Transpa"
  "rent&lt;/i&gt;</property><property name=\"use_markup\">True</property><"
  "/object><packing><property name=\"expand\">False</property><property na"
  "me=\"fill\">False</property><property name=\"position\">0</property></p"
  "acking></child><child><object class=\"GtkScale\" id=\"resize_opacity_sc"
  "ale\"><property name=\"visible\">True</property><property name=\"can_fo"
  "cus\">True</property><property name=\"orientation\">horizontal</propert"
  "y><property name=\"adjustment\">adjustment5</property><property name=\""
  "restrict_to_fill_level\">False</property><property name=\"draw_value\">"
  "False</property></object><packing><property name=\"expand\">True</prope"
  "rty><property name=\"fill\">True</property><property name=\"position\">"
  "1</property></packing></child><child><object class=\"GtkLabel\" id=\"la"
  "bel24\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"label\" translatable=\"yes\">&"
  "lt;i&gt;Opaque&lt;/i&gt;</property><property name=\"use_markup\">True</"
  "property></object><packing><property name=\"expand\">False</property><p"
  "roperty name=\"fill\">False</property><property name=\"position\">2</pr"
  "operty></packing></child></object></child></object><packing><property n"
  "ame=\"expand\">False</property><property name=\"fill\">False</property>"
  "<property name=\"position\">15</property></packing></child><child><obje"
  "ct class=\"GtkLabel\" id=\"label16\"><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">False</property><property name=\"x"
  "align\">0</property><property name=\"label\" translatable=\"yes\">Opaci"
  "ty of popup wi_ndows:</property><property name=\"use_underline\">True</"
  "property><property name=\"mnemonic_widget\">popup_opacity_scale</proper"
  "ty></object><packing><property name=\"expand\">False</property><propert"
  "y name=\"fill\">False</property><property name=\"position\">16</propert"
  "y></packing></child><child><object class=\"GtkAlignment\" id=\"alignmen"
  "t9\"><property name=\"visible\">True</property><property name=\"can_foc"
  "us\">False</property><property name=\"top_padding\">2</property><proper"
  "ty name=\"left_padding\">12</property><child><object class=\"GtkBox\" i"
  "d=\"hbox7\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"orientation\">horizontal</"
  "property><child><object class=\"GtkLabel\" id=\"label25\"><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">False</proper"
  "ty><property name=\"label\" translatable=\"yes\">&lt;i&gt;Transparent&l"
  "t;/i&gt;</property><property name=\"use_markup\">True</property></objec"
  "t><packing><property name=\"expand\">False</property><property name=\"f"
  "ill\">False</property><property name=\"position\">0</property></packing"
  "></child><child><object class=\"GtkScale\" id=\"popup_opacity_scale\"><"
  "property name=\"visible\">True</property><property name=\"can_focus\">T"
  "rue</property><property name=\"orientation\">horizontal</property><prop"
  "erty name=\"adjustment\">adjustment6</property><property name=\"restric"
  "t_to_fill_level\">False</property><property name=\"draw_value\">False</"
  "property></object><packing><property name=\"expand\">True</property><pr"
  "operty name=\"fill\">True</property><property name=\"position\">1</prop"
  "erty></packing></child><child><object class=\"GtkLabel\" id=\"label26\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"label\" translatable=\"yes\">&lt;i&gt"
  ";Opaque&lt;/i&gt;</property><property name=\"use_markup\">True</propert"
  "y></object><packing><property name=\"expand\">False</property><property"
  " name=\"fill\">False</property><property name=\"position\">2</property>"
  "</packing></child></object></child></object><packing><property name=\"e"
  "xpand\">False</property><property name=\"fill\">False</property><proper"
  "ty name=\"position\">17</property></packing></child></object></child></"
  "object></child></object></child></object><packing><property name=\"expa"
  "nd\">True</property><property name=\"fill\">True</property><property na"
  "me=\"position\">1</property></packing></child></object><packing><proper"
  "ty name=\"position\">5</property></packing></child><child type=\"tab\">"
  "<object class=\"GtkLabel\" id=\"label6\"><property name=\"visible\">Tru"
  "e</property><property name=\"can_focus\">False</property><property name"
  "=\"label\" translatable=\"yes\">C_ompositor</property><property name=\""
  "use_underline\">True</property></object><packing><property name=\"posit"
  "ion\">5</property><property name=\"tab_fill\">False</property></packing"
  "></child></object><packing><property name=\"expand\">True</property><pr"
  "operty name=\"fill\">True</property><property name=\"position\">1</prop"
  "erty></packing></child></object></child><action-widgets><action-widget "
  "response=\"-11\">button2</action-widget><action-widget response=\"0\">b"
  "utton1</action-widget></action-widgets></object></interface>"
};

static const unsigned tweaks_dialog_ui_length = 48217u;

