/* automatically generated from xfwm4-dialog.glade */
#ifdef __SUNPRO_C
#pragma align 4 (xfwm4_dialog_ui)
#endif
#ifdef __GNUC__
static const char xfwm4_dialog_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char xfwm4_dialog_ui[] =
#endif
{
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?><interface><requires lib=\"gt"
  "k+\" version=\"3.20\"/><object class=\"GtkAdjustment\" id=\"adjustment1"
  "\"><property name=\"lower\">5</property><property name=\"upper\">2000</"
  "property><property name=\"value\">5</property><property name=\"step_inc"
  "rement\">1</property><property name=\"page_increment\">10</property></o"
  "bject><object class=\"GtkAdjustment\" id=\"adjustment2\"><property name"
  "=\"lower\">5</property><property name=\"upper\">2000</property><propert"
  "y name=\"value\">5</property><property name=\"step_increment\">1</prope"
  "rty><property name=\"page_increment\">10</property></object><object cla"
  "ss=\"GtkAdjustment\" id=\"adjustment3\"><property name=\"lower\">5</pro"
  "perty><property name=\"upper\">100</property><property name=\"value\">2"
  "0</property><property name=\"step_increment\">1</property><property nam"
  "e=\"page_increment\">10</property></object><object class=\"GtkAdjustmen"
  "t\" id=\"adjustment4\"><property name=\"lower\">5</property><property n"
  "ame=\"upper\">100</property><property name=\"value\">20</property><prop"
  "erty name=\"step_increment\">1</property><property name=\"page_incremen"
  "t\">10</property></object><object class=\"GtkImage\" id=\"image7\"><pro"
  "perty name=\"visible\">True</property><property name=\"can_focus\">Fals"
  "e</property><property name=\"icon_name\">window-close-symbolic</propert"
  "y></object><object class=\"GtkImage\" id=\"image8\"><property name=\"vi"
  "sible\">True</property><property name=\"can_focus\">False</property><pr"
  "operty name=\"icon_name\">help-browser-symbolic</property></object><obj"
  "ect class=\"GtkImage\" id=\"image9\"><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">False</property><property name=\"i"
  "con_name\">document-edit-symbolic</property></object><object class=\"Gt"
  "kImage\" id=\"image10\"><property name=\"visible\">True</property><prop"
  "erty name=\"can_focus\">False</property><property name=\"icon_name\">ed"
  "it-clear-symbolic</property></object><object class=\"GtkImage\" id=\"im"
  "age11\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"icon_name\">document-revert-sy"
  "mbolic</property></object><object class=\"XfceTitledDialog\" id=\"main-"
  "dialog\"><property name=\"can_focus\">False</property><property name=\""
  "title\" translatable=\"yes\">Window Manager</property><property name=\""
  "window_position\">center-on-parent</property><property name=\"icon_name"
  "\">org.xfce.xfwm4</property><property name=\"type_hint\">dialog</proper"
  "ty><child internal-child=\"vbox\"><object class=\"GtkBox\" id=\"main-vb"
  "ox\"><property name=\"visible\">True</property><property name=\"can_foc"
  "us\">False</property><property name=\"orientation\">vertical</property>"
  "<child internal-child=\"action_area\"><object class=\"GtkButtonBox\" id"
  "=\"dialog-action_area1\"><property name=\"visible\">True</property><pro"
  "perty name=\"can_focus\">False</property><property name=\"layout_style\""
  ">end</property><child><object class=\"GtkButton\" id=\"help_button\"><p"
  "roperty name=\"label\" translatable=\"yes\">_Help</property><property n"
  "ame=\"use_action_appearance\">False</property><property name=\"visible\""
  ">True</property><property name=\"can_focus\">True</property><property n"
  "ame=\"receives_default\">True</property><property name=\"image\">image8"
  "</property><property name=\"use_underline\">True</property></object><pa"
  "cking><property name=\"expand\">False</property><property name=\"fill\""
  ">False</property><property name=\"position\">0</property><property name"
  "=\"secondary\">True</property></packing></child><child><object class=\""
  "GtkButton\" id=\"close_button\"><property name=\"label\" translatable=\""
  "yes\">_Close</property><property name=\"use_action_appearance\">False</"
  "property><property name=\"visible\">True</property><property name=\"can"
  "_focus\">True</property><property name=\"receives_default\">True</prope"
  "rty><property name=\"image\">image7</property><property name=\"use_unde"
  "rline\">True</property></object><packing><property name=\"expand\">Fals"
  "e</property><property name=\"fill\">False</property><property name=\"po"
  "sition\">1</property></packing></child></object><packing><property name"
  "=\"expand\">False</property><property name=\"fill\">True</property><pro"
  "perty name=\"pack_type\">end</property><property name=\"position\">0</p"
  "roperty></packing></child><child><object class=\"GtkNotebook\" id=\"plu"
  "g-child\"><property name=\"visible\">True</property><property name=\"ca"
  "n_focus\">True</property><property name=\"border_width\">6</property><c"
  "hild><object class=\"GtkBox\" id=\"hbox7\"><property name=\"visible\">T"
  "rue</property><property name=\"can_focus\">False</property><property na"
  "me=\"orientation\">horizontal</property><property name=\"border_width\""
  ">12</property><property name=\"spacing\">12</property><child><object cl"
  "ass=\"GtkFrame\" id=\"frame13\"><property name=\"visible\">True</proper"
  "ty><property name=\"can_focus\">False</property><property name=\"label_"
  "xalign\">0</property><property name=\"shadow_type\">none</property><chi"
  "ld><object class=\"GtkAlignment\" id=\"alignment11\"><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">False</property><p"
  "roperty name=\"top_padding\">6</property><property name=\"left_padding\""
  ">12</property><child><object class=\"GtkScrolledWindow\" id=\"scrolledw"
  "indow3\"><property name=\"width_request\">200</property><property name="
  "\"visible\">True</property><property name=\"can_focus\">True</property>"
  "<property name=\"hscrollbar_policy\">automatic</property><property name"
  "=\"vscrollbar_policy\">automatic</property><property name=\"shadow_type"
  "\">in</property><child><object class=\"GtkTreeView\" id=\"theme_name_tr"
  "eeview\"><property name=\"visible\">True</property><property name=\"can"
  "_focus\">True</property><property name=\"headers_visible\">False</prope"
  "rty></object></child></object></child></object></child><child type=\"la"
  "bel\"><object class=\"GtkLabel\" id=\"label32\"><property name=\"visibl"
  "e\">True</property><property name=\"can_focus\">False</property><proper"
  "ty name=\"label\" translatable=\"yes\">&lt;b&gt;The_me&lt;/b&gt;</prope"
  "rty><property name=\"use_markup\">True</property><property name=\"use_u"
  "nderline\">True</property></object></child></object><packing><property "
  "name=\"expand\">True</property><property name=\"fill\">True</property><"
  "property name=\"position\">0</property></packing></child><child><object"
  " class=\"GtkBox\" id=\"vbox12\"><property name=\"visible\">True</proper"
  "ty><property name=\"can_focus\">False</property><property name=\"orient"
  "ation\">vertical</property><property name=\"spacing\">6</property><chil"
  "d><object class=\"GtkFrame\" id=\"frame10\"><property name=\"visible\">"
  "True</property><property name=\"can_focus\">False</property><property n"
  "ame=\"label_xalign\">0</property><property name=\"shadow_type\">none</p"
  "roperty><child><object class=\"GtkAlignment\" id=\"alignment14\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">False<"
  "/property><property name=\"border_width\">6</property><property name=\""
  "bottom_padding\">6</property><property name=\"left_padding\">12</proper"
  "ty><child><object class=\"GtkFontButton\" id=\"title_font_button\"><pro"
  "perty name=\"use_action_appearance\">False</property><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">True</property><pr"
  "operty name=\"receives_default\">True</property></object></child></obje"
  "ct></child><child type=\"label\"><object class=\"GtkLabel\" id=\"label2"
  "3\"><property name=\"visible\">True</property><property name=\"can_focu"
  "s\">False</property><property name=\"label\" translatable=\"yes\">&lt;b"
  "&gt;Title fon_t&lt;/b&gt;</property><property name=\"use_markup\">True<"
  "/property><property name=\"use_underline\">True</property><property nam"
  "e=\"mnemonic_widget\">title_font_button</property></object></child></ob"
  "ject><packing><property name=\"expand\">False</property><property name="
  "\"fill\">False</property><property name=\"position\">0</property></pack"
  "ing></child><child><object class=\"GtkFrame\" id=\"frame11\"><property "
  "name=\"visible\">True</property><property name=\"can_focus\">False</pro"
  "perty><property name=\"label_xalign\">0</property><property name=\"shad"
  "ow_type\">none</property><child><object class=\"GtkAlignment\" id=\"tit"
  "le_align_box\"><property name=\"visible\">True</property><property name"
  "=\"can_focus\">False</property><property name=\"border_width\">6</prope"
  "rty><property name=\"bottom_padding\">6</property><property name=\"left"
  "_padding\">12</property><child><object class=\"GtkComboBox\" id=\"title"
  "_align_combo\"><property name=\"visible\">True</property><property name"
  "=\"can_focus\">False</property></object></child></object></child><child"
  " type=\"label\"><object class=\"GtkLabel\" id=\"label24\"><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">False</proper"
  "ty><property name=\"label\" translatable=\"yes\">&lt;b&gt;Title _alignm"
  "ent&lt;/b&gt;</property><property name=\"use_markup\">True</property><p"
  "roperty name=\"use_underline\">True</property><property name=\"mnemonic"
  "_widget\">title_align_combo</property></object></child></object><packin"
  "g><property name=\"expand\">False</property><property name=\"fill\">Fal"
  "se</property><property name=\"position\">1</property></packing></child>"
  "<child><object class=\"GtkFrame\" id=\"frame12\"><property name=\"visib"
  "le\">True</property><property name=\"can_focus\">False</property><prope"
  "rty name=\"label_xalign\">0</property><property name=\"shadow_type\">no"
  "ne</property><child><object class=\"GtkAlignment\" id=\"button_layout_b"
  "ox\"><property name=\"visible\">True</property><property name=\"can_foc"
  "us\">False</property><property name=\"bottom_padding\">6</property><pro"
  "perty name=\"left_padding\">12</property><child><object class=\"GtkBox\""
  " id=\"vbox13\"><property name=\"visible\">True</property><property name"
  "=\"can_focus\">False</property><property name=\"orientation\">vertical<"
  "/property><property name=\"border_width\">6</property><property name=\""
  "spacing\">6</property><child><object class=\"GtkLabel\" id=\"label26\">"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "False</property><property name=\"xalign\">0</property><property name=\""
  "label\" translatable=\"yes\">Click and drag the buttons to change the l"
  "ayout</property></object><packing><property name=\"expand\">True</prope"
  "rty><property name=\"fill\">True</property><property name=\"position\">"
  "0</property></packing></child><child><object class=\"GtkFrame\" id=\"ac"
  "tive-frame\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"label_xalign\">0</property"
  "><child><object class=\"GtkAlignment\" id=\"alignment17\"><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">False</proper"
  "ty><property name=\"border_width\">6</property><child><object class=\"G"
  "tkBox\" id=\"active-box\"><property name=\"visible\">True</property><pr"
  "operty name=\"can_focus\">False</property><property name=\"orientation\""
  ">horizontal</property><property name=\"spacing\">6</property><child><ob"
  "ject class=\"GtkButton\" id=\"button-layout-|\"><property name=\"label\""
  " translatable=\"yes\">Title</property><property name=\"use_action_appea"
  "rance\">False</property><property name=\"visible\">True</property><prop"
  "erty name=\"can_focus\">True</property><property name=\"receives_defaul"
  "t\">True</property><property name=\"tooltip_text\" translatable=\"yes\""
  ">The window title cannot be removed</property></object><packing><proper"
  "ty name=\"expand\">False</property><property name=\"fill\">False</prope"
  "rty><property name=\"position\">0</property></packing></child></object>"
  "</child></object></child><child type=\"label\"><object class=\"GtkLabel"
  "\" id=\"label27\"><property name=\"visible\">True</property><property n"
  "ame=\"can_focus\">False</property><property name=\"label\" translatable"
  "=\"yes\">Active</property><property name=\"use_markup\">True</property>"
  "</object></child></object><packing><property name=\"expand\">True</prop"
  "erty><property name=\"fill\">True</property><property name=\"position\""
  ">1</property></packing></child><child><object class=\"GtkFrame\" id=\"h"
  "idden-frame\"><property name=\"visible\">True</property><property name="
  "\"can_focus\">False</property><property name=\"label_xalign\">0</proper"
  "ty><child><object class=\"GtkAlignment\" id=\"alignment18\"><property n"
  "ame=\"visible\">True</property><property name=\"can_focus\">False</prop"
  "erty><property name=\"border_width\">6</property><child><object class=\""
  "GtkBox\" id=\"hidden-box\"><property name=\"visible\">True</property><p"
  "roperty name=\"can_focus\">False</property><property name=\"orientation"
  "\">horizontal</property><property name=\"spacing\">6</property><child><"
  "object class=\"GtkButton\" id=\"button-layout-O\"><property name=\"use_"
  "action_appearance\">False</property><property name=\"visible\">True</pr"
  "operty><property name=\"can_focus\">True</property><property name=\"rec"
  "eives_default\">True</property><property name=\"tooltip_text\" translat"
  "able=\"yes\">Menu</property><child><object class=\"GtkImage\" id=\"imag"
  "e1\"><property name=\"visible\">True</property><property name=\"can_foc"
  "us\">False</property><property name=\"icon_name\">xfce-wm-menu</propert"
  "y></object></child></object><packing><property name=\"expand\">False</p"
  "roperty><property name=\"fill\">False</property><property name=\"positi"
  "on\">0</property></packing></child><child><object class=\"GtkButton\" i"
  "d=\"button-layout-T\"><property name=\"use_action_appearance\">False</p"
  "roperty><property name=\"visible\">True</property><property name=\"can_"
  "focus\">True</property><property name=\"receives_default\">True</proper"
  "ty><property name=\"tooltip_text\" translatable=\"yes\">Stick</property"
  "><child><object class=\"GtkImage\" id=\"image2\"><property name=\"visib"
  "le\">True</property><property name=\"can_focus\">False</property><prope"
  "rty name=\"icon_name\">xfce-wm-stick</property></object></child></objec"
  "t><packing><property name=\"expand\">False</property><property name=\"f"
  "ill\">False</property><property name=\"position\">1</property></packing"
  "></child><child><object class=\"GtkButton\" id=\"button-layout-S\"><pro"
  "perty name=\"use_action_appearance\">False</property><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">True</property><pr"
  "operty name=\"receives_default\">True</property><property name=\"toolti"
  "p_text\" translatable=\"yes\">Shade</property><child><object class=\"Gt"
  "kImage\" id=\"image3\"><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">False</property><property name=\"icon_name\">xfc"
  "e-wm-shade</property></object></child></object><packing><property name="
  "\"expand\">False</property><property name=\"fill\">False</property><pro"
  "perty name=\"position\">2</property></packing></child><child><object cl"
  "ass=\"GtkButton\" id=\"button-layout-H\"><property name=\"use_action_ap"
  "pearance\">False</property><property name=\"visible\">True</property><p"
  "roperty name=\"can_focus\">True</property><property name=\"receives_def"
  "ault\">True</property><property name=\"tooltip_text\" translatable=\"ye"
  "s\">Minimize</property><child><object class=\"GtkImage\" id=\"image4\">"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "False</property><property name=\"icon_name\">xfce-wm-minimize</property"
  "></object></child></object><packing><property name=\"expand\">False</pr"
  "operty><property name=\"fill\">False</property><property name=\"positio"
  "n\">3</property></packing></child><child><object class=\"GtkButton\" id"
  "=\"button-layout-M\"><property name=\"use_action_appearance\">False</pr"
  "operty><property name=\"visible\">True</property><property name=\"can_f"
  "ocus\">True</property><property name=\"receives_default\">True</propert"
  "y><property name=\"tooltip_text\" translatable=\"yes\">Maximize</proper"
  "ty><child><object class=\"GtkImage\" id=\"image5\"><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">False</property><pro"
  "perty name=\"icon_name\">xfce-wm-maximize</property></object></child></"
  "object><packing><property name=\"expand\">False</property><property nam"
  "e=\"fill\">False</property><property name=\"position\">4</property></pa"
  "cking></child><child><object class=\"GtkButton\" id=\"button-layout-C\""
  "><property name=\"use_action_appearance\">False</property><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">True</propert"
  "y><property name=\"receives_default\">True</property><property name=\"t"
  "ooltip_text\" translatable=\"yes\">Close</property><child><object class"
  "=\"GtkImage\" id=\"image6\"><property name=\"visible\">True</property><"
  "property name=\"can_focus\">False</property><property name=\"icon_name\""
  ">xfce-wm-close</property></object></child></object><packing><property n"
  "ame=\"expand\">False</property><property name=\"fill\">False</property>"
  "<property name=\"position\">5</property></packing></child></object></ch"
  "ild></object></child><child type=\"label\"><object class=\"GtkLabel\" i"
  "d=\"label28\"><property name=\"visible\">True</property><property name="
  "\"can_focus\">False</property><property name=\"label\" translatable=\"y"
  "es\">Hidden</property><property name=\"use_markup\">True</property></ob"
  "ject></child></object><packing><property name=\"expand\">True</property"
  "><property name=\"fill\">True</property><property name=\"position\">2</"
  "property></packing></child></object></child></object></child><child typ"
  "e=\"label\"><object class=\"GtkLabel\" id=\"label25\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">False</property><"
  "property name=\"label\" translatable=\"yes\">&lt;b&gt;Button layout&lt;"
  "/b&gt;</property><property name=\"use_markup\">True</property></object>"
  "</child></object><packing><property name=\"expand\">False</property><pr"
  "operty name=\"fill\">False</property><property name=\"position\">2</pro"
  "perty></packing></child></object><packing><property name=\"expand\">Tru"
  "e</property><property name=\"fill\">True</property><property name=\"pos"
  "ition\">1</property></packing></child></object></child><child type=\"ta"
  "b\"><object class=\"GtkLabel\" id=\"label1\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property><property "
  "name=\"label\" translatable=\"yes\">_Style</property><property name=\"u"
  "se_underline\">True</property></object><packing><property name=\"tab_fi"
  "ll\">False</property></packing></child><child><object class=\"GtkBox\" "
  "id=\"vbox10\"><property name=\"visible\">True</property><property name="
  "\"can_focus\">False</property><property name=\"orientation\">vertical</"
  "property><property name=\"border_width\">12</property><property name=\""
  "spacing\">6</property><child><object class=\"GtkLabel\" id=\"label22\">"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "False</property><property name=\"xalign\">0</property><property name=\""
  "label\" translatable=\"yes\">Define shortcuts to perform _window manage"
  "r actions:</property><property name=\"use_markup\">True</property><prop"
  "erty name=\"use_underline\">True</property></object><packing><property "
  "name=\"expand\">False</property><property name=\"fill\">False</property"
  "><property name=\"position\">0</property></packing></child><child><obje"
  "ct class=\"GtkBox\" id=\"vbox11\"><property name=\"visible\">True</prop"
  "erty><property name=\"can_focus\">False</property><property name=\"orie"
  "ntation\">vertical</property><property name=\"spacing\">6</property><ch"
  "ild><object class=\"GtkScrolledWindow\" id=\"scrolledwindow2\"><propert"
  "y name=\"visible\">True</property><property name=\"can_focus\">True</pr"
  "operty><property name=\"hscrollbar_policy\">never</property><property n"
  "ame=\"vscrollbar_policy\">automatic</property><property name=\"shadow_t"
  "ype\">in</property><child><object class=\"GtkTreeView\" id=\"shortcuts_"
  "treeview\"><property name=\"visible\">True</property><property name=\"c"
  "an_focus\">True</property></object></child></object><packing><property "
  "name=\"expand\">True</property><property name=\"fill\">True</property><"
  "property name=\"position\">0</property></packing></child><child><placeh"
  "older/></child></object><packing><property name=\"expand\">True</proper"
  "ty><property name=\"fill\">True</property><property name=\"position\">1"
  "</property></packing></child><child><object class=\"GtkAlignment\" id=\""
  "alignment15\"><property name=\"visible\">True</property><property name="
  "\"can_focus\">False</property><property name=\"yalign\">0</property><pr"
  "operty name=\"xscale\">0</property><child><object class=\"GtkBox\" id=\""
  "hbox5\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"orientation\">horizontal</prop"
  "erty><property name=\"spacing\">6</property><child><object class=\"GtkB"
  "utton\" id=\"shortcuts_edit_button\"><property name=\"label\" translata"
  "ble=\"yes\">_Edit</property><property name=\"use_action_appearance\">Fa"
  "lse</property><property name=\"visible\">True</property><property name="
  "\"can_focus\">True</property><property name=\"receives_default\">True</"
  "property><property name=\"use_underline\">True</property><property name"
  "=\"image\">image9</property></object><packing><property name=\"expand\""
  ">False</property><property name=\"fill\">False</property><property name"
  "=\"position\">0</property></packing></child><child><object class=\"GtkB"
  "utton\" id=\"shortcuts_clear_button\"><property name=\"label\" translat"
  "able=\"yes\">_Clear</property><property name=\"use_action_appearance\">"
  "False</property><property name=\"visible\">True</property><property nam"
  "e=\"can_focus\">True</property><property name=\"receives_default\">True"
  "</property><property name=\"use_underline\">True</property><property na"
  "me=\"image\">image10</property></object><packing><property name=\"expan"
  "d\">False</property><property name=\"fill\">False</property><property n"
  "ame=\"position\">1</property></packing></child><child><object class=\"G"
  "tkButton\" id=\"shortcuts_reset_button\"><property name=\"label\" trans"
  "latable=\"yes\">_Reset to Defaults</property><property name=\"use_actio"
  "n_appearance\">False</property><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">True</property><property name=\"receives"
  "_default\">True</property><property name=\"use_underline\">True</proper"
  "ty><property name=\"image\">image11</property></object><packing><proper"
  "ty name=\"expand\">False</property><property name=\"fill\">False</prope"
  "rty><property name=\"padding\">15</property><property name=\"pack_type\""
  ">end</property><property name=\"position\">2</property></packing></chil"
  "d></object></child></object><packing><property name=\"expand\">False</p"
  "roperty><property name=\"fill\">False</property><property name=\"positi"
  "on\">2</property></packing></child></object><packing><property name=\"p"
  "osition\">1</property></packing></child><child type=\"tab\"><object cla"
  "ss=\"GtkLabel\" id=\"label2\"><property name=\"visible\">True</property"
  "><property name=\"can_focus\">False</property><property name=\"label\" "
  "translatable=\"yes\">_Keyboard</property><property name=\"use_underline"
  "\">True</property></object><packing><property name=\"position\">1</prop"
  "erty><property name=\"tab_fill\">False</property></packing></child><chi"
  "ld><object class=\"GtkBox\" id=\"vbox7\"><property name=\"visible\">Tru"
  "e</property><property name=\"can_focus\">False</property><property name"
  "=\"orientation\">vertical</property><property name=\"border_width\">12<"
  "/property><property name=\"spacing\">6</property><child><object class=\""
  "GtkFrame\" id=\"frame5\"><property name=\"visible\">True</property><pro"
  "perty name=\"can_focus\">False</property><property name=\"label_xalign\""
  ">0</property><property name=\"shadow_type\">none</property><child><obje"
  "ct class=\"GtkAlignment\" id=\"alignment7\"><property name=\"visible\">"
  "True</property><property name=\"can_focus\">False</property><property n"
  "ame=\"border_width\">6</property><property name=\"bottom_padding\">6</p"
  "roperty><property name=\"left_padding\">12</property><child><object cla"
  "ss=\"GtkBox\" id=\"vbox14\"><property name=\"visible\">True</property><"
  "property name=\"can_focus\">False</property><property name=\"orientatio"
  "n\">vertical</property><property name=\"spacing\">6</property><child><o"
  "bject class=\"GtkBox\" id=\"hbox3\"><property name=\"visible\">True</pr"
  "operty><property name=\"can_focus\">False</property><property name=\"or"
  "ientation\">horizontal</property><property name=\"spacing\">12</propert"
  "y><property name=\"homogeneous\">True</property><child><object class=\""
  "GtkRadioButton\" id=\"click_to_focus_radio\"><property name=\"label\" t"
  "ranslatable=\"yes\">Click to foc_us</property><property name=\"use_acti"
  "on_appearance\">False</property><property name=\"visible\">True</proper"
  "ty><property name=\"can_focus\">True</property><property name=\"receive"
  "s_default\">False</property><property name=\"use_underline\">True</prop"
  "erty><property name=\"draw_indicator\">True</property></object><packing"
  "><property name=\"expand\">True</property><property name=\"fill\">True<"
  "/property><property name=\"position\">0</property></packing></child><ch"
  "ild><object class=\"GtkRadioButton\" id=\"focus_follows_mouse_radio\"><"
  "property name=\"label\" translatable=\"yes\">Focus follows _mouse</prop"
  "erty><property name=\"use_action_appearance\">False</property><property"
  " name=\"visible\">True</property><property name=\"can_focus\">True</pro"
  "perty><property name=\"receives_default\">False</property><property nam"
  "e=\"use_underline\">True</property><property name=\"draw_indicator\">Tr"
  "ue</property><property name=\"group\">click_to_focus_radio</property></"
  "object><packing><property name=\"expand\">True</property><property name"
  "=\"fill\">True</property><property name=\"position\">1</property></pack"
  "ing></child></object><packing><property name=\"expand\">True</property>"
  "<property name=\"fill\">True</property><property name=\"position\">0</p"
  "roperty></packing></child><child><object class=\"GtkLabel\" id=\"label2"
  "9\"><property name=\"visible\">True</property><property name=\"can_focu"
  "s\">False</property><property name=\"xalign\">0</property><property nam"
  "e=\"label\" translatable=\"yes\">_Delay before window receives focus:</"
  "property><property name=\"use_markup\">True</property><property name=\""
  "use_underline\">True</property><property name=\"mnemonic_widget\">focus"
  "_delay_scale</property></object><packing><property name=\"expand\">True"
  "</property><property name=\"fill\">True</property><property name=\"posi"
  "tion\">1</property></packing></child><child><object class=\"GtkAlignmen"
  "t\" id=\"alignment19\"><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">False</property><property name=\"left_padding\">"
  "12</property><child><object class=\"GtkBox\" id=\"focus_delay_hbox\"><p"
  "roperty name=\"visible\">True</property><property name=\"can_focus\">Fa"
  "lse</property><property name=\"orientation\">horizontal</property><prop"
  "erty name=\"spacing\">12</property><child><object class=\"GtkLabel\" id"
  "=\"label30\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"label\" translatable=\"yes"
  "\" comments=\"Focus delay\">&lt;i&gt;Short&lt;/i&gt;</property><propert"
  "y name=\"use_markup\">True</property></object><packing><property name=\""
  "expand\">False</property><property name=\"fill\">False</property><prope"
  "rty name=\"position\">0</property></packing></child><child><object clas"
  "s=\"GtkScale\" id=\"focus_delay_scale\"><property name=\"visible\">True"
  "</property><property name=\"can_focus\">True</property><property name=\""
  "orientation\">horizontal</property><property name=\"adjustment\">adjust"
  "ment1</property><property name=\"draw_value\">False</property></object>"
  "<packing><property name=\"expand\">True</property><property name=\"fill"
  "\">True</property><property name=\"position\">1</property></packing></c"
  "hild><child><object class=\"GtkLabel\" id=\"label31\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">False</property><"
  "property name=\"label\" translatable=\"yes\" comments=\"Focus delay\">&"
  "lt;i&gt;Long&lt;/i&gt;</property><property name=\"use_markup\">True</pr"
  "operty></object><packing><property name=\"expand\">False</property><pro"
  "perty name=\"fill\">False</property><property name=\"position\">2</prop"
  "erty></packing></child></object></child></object><packing><property nam"
  "e=\"expand\">True</property><property name=\"fill\">True</property><pro"
  "perty name=\"position\">2</property></packing></child></object></child>"
  "</object></child><child type=\"label\"><object class=\"GtkLabel\" id=\""
  "label15\"><property name=\"visible\">True</property><property name=\"ca"
  "n_focus\">False</property><property name=\"label\" translatable=\"yes\""
  ">&lt;b&gt;Focus model&lt;/b&gt;</property><property name=\"use_markup\""
  ">True</property></object></child></object><packing><property name=\"exp"
  "and\">False</property><property name=\"fill\">False</property><property"
  " name=\"position\">0</property></packing></child><child><object class=\""
  "GtkFrame\" id=\"frame6\"><property name=\"visible\">True</property><pro"
  "perty name=\"can_focus\">False</property><property name=\"label_xalign\""
  ">0</property><property name=\"shadow_type\">none</property><child><obje"
  "ct class=\"GtkAlignment\" id=\"alignment8\"><property name=\"visible\">"
  "True</property><property name=\"can_focus\">False</property><property n"
  "ame=\"border_width\">6</property><property name=\"bottom_padding\">6</p"
  "roperty><property name=\"left_padding\">12</property><child><object cla"
  "ss=\"GtkCheckButton\" id=\"focus_new_check\"><property name=\"label\" t"
  "ranslatable=\"yes\">Automatically give focus to _newly created windows<"
  "/property><property name=\"use_action_appearance\">False</property><pro"
  "perty name=\"visible\">True</property><property name=\"can_focus\">True"
  "</property><property name=\"receives_default\">False</property><propert"
  "y name=\"use_underline\">True</property><property name=\"draw_indicator"
  "\">True</property></object></child></object></child><child type=\"label"
  "\"><object class=\"GtkLabel\" id=\"label16\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property><property "
  "name=\"label\" translatable=\"yes\">&lt;b&gt;New window focus&lt;/b&gt;"
  "</property><property name=\"use_markup\">True</property></object></chil"
  "d></object><packing><property name=\"expand\">False</property><property"
  " name=\"fill\">False</property><property name=\"position\">1</property>"
  "</packing></child><child><object class=\"GtkFrame\" id=\"frame7\"><prop"
  "erty name=\"visible\">True</property><property name=\"can_focus\">False"
  "</property><property name=\"label_xalign\">0</property><property name=\""
  "shadow_type\">none</property><child><object class=\"GtkAlignment\" id=\""
  "alignment9\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"border_width\">6</property"
  "><property name=\"bottom_padding\">6</property><property name=\"left_pa"
  "dding\">12</property><child><object class=\"GtkBox\" id=\"vbox8\"><prop"
  "erty name=\"visible\">True</property><property name=\"can_focus\">False"
  "</property><property name=\"orientation\">vertical</property><property "
  "name=\"spacing\">6</property><child><object class=\"GtkCheckButton\" id"
  "=\"raise_on_focus_check\"><property name=\"label\" translatable=\"yes\""
  ">Automatically _raise windows when they receive focus</property><proper"
  "ty name=\"use_action_appearance\">False</property><property name=\"visi"
  "ble\">True</property><property name=\"can_focus\">True</property><prope"
  "rty name=\"receives_default\">False</property><property name=\"use_unde"
  "rline\">True</property><property name=\"draw_indicator\">True</property"
  "></object><packing><property name=\"expand\">True</property><property n"
  "ame=\"fill\">True</property><property name=\"position\">0</property></p"
  "acking></child><child><object class=\"GtkBox\" id=\"vbox9\"><property n"
  "ame=\"visible\">True</property><property name=\"can_focus\">False</prop"
  "erty><property name=\"orientation\">vertical</property><property name=\""
  "spacing\">6</property><child><object class=\"GtkLabel\" id=\"label19\">"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "False</property><property name=\"xalign\">0</property><property name=\""
  "label\" translatable=\"yes\">Delay _before raising focused window:</pro"
  "perty><property name=\"use_underline\">True</property><property name=\""
  "mnemonic_widget\">focus_raise_delay_scale</property></object><packing><"
  "property name=\"expand\">False</property><property name=\"fill\">False<"
  "/property><property name=\"position\">0</property></packing></child><ch"
  "ild><object class=\"GtkAlignment\" id=\"alignment1\"><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">False</property><p"
  "roperty name=\"left_padding\">12</property><child><object class=\"GtkBo"
  "x\" id=\"hbox4\"><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">False</property><property name=\"orientation\">horizon"
  "tal</property><property name=\"spacing\">12</property><child><object cl"
  "ass=\"GtkLabel\" id=\"label20\"><property name=\"visible\">True</proper"
  "ty><property name=\"can_focus\">False</property><property name=\"label\""
  " translatable=\"yes\" comments=\"Raise focus delay\">&lt;i&gt;Short&lt;"
  "/i&gt;</property><property name=\"use_markup\">True</property></object>"
  "<packing><property name=\"expand\">False</property><property name=\"fil"
  "l\">False</property><property name=\"position\">0</property></packing><"
  "/child><child><object class=\"GtkScale\" id=\"focus_raise_delay_scale\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">True</property><property name=\"orientation\">horizontal</property><pr"
  "operty name=\"adjustment\">adjustment2</property><property name=\"draw_"
  "value\">False</property></object><packing><property name=\"expand\">Tru"
  "e</property><property name=\"fill\">True</property><property name=\"pos"
  "ition\">1</property></packing></child><child><object class=\"GtkLabel\""
  " id=\"label21\"><property name=\"visible\">True</property><property nam"
  "e=\"can_focus\">False</property><property name=\"label\" translatable=\""
  "yes\" comments=\"Raise focus delay\">&lt;i&gt;Long&lt;/i&gt;</property>"
  "<property name=\"use_markup\">True</property></object><packing><propert"
  "y name=\"expand\">False</property><property name=\"fill\">False</proper"
  "ty><property name=\"position\">2</property></packing></child></object><"
  "/child></object><packing><property name=\"expand\">True</property><prop"
  "erty name=\"fill\">True</property><property name=\"position\">1</proper"
  "ty></packing></child></object><packing><property name=\"expand\">True</"
  "property><property name=\"fill\">True</property><property name=\"positi"
  "on\">1</property></packing></child></object></child></object></child><c"
  "hild type=\"label\"><object class=\"GtkLabel\" id=\"label17\"><property"
  " name=\"visible\">True</property><property name=\"can_focus\">False</pr"
  "operty><property name=\"label\" translatable=\"yes\">&lt;b&gt;Raise on "
  "focus&lt;/b&gt;</property><property name=\"use_markup\">True</property>"
  "</object></child></object><packing><property name=\"expand\">False</pro"
  "perty><property name=\"fill\">False</property><property name=\"position"
  "\">2</property></packing></child><child><object class=\"GtkFrame\" id=\""
  "frame8\"><property name=\"visible\">True</property><property name=\"can"
  "_focus\">False</property><property name=\"label_xalign\">0</property><p"
  "roperty name=\"shadow_type\">none</property><child><object class=\"GtkA"
  "lignment\" id=\"alignment10\"><property name=\"visible\">True</property"
  "><property name=\"can_focus\">False</property><property name=\"border_w"
  "idth\">6</property><property name=\"bottom_padding\">6</property><prope"
  "rty name=\"left_padding\">12</property><child><object class=\"GtkCheckB"
  "utton\" id=\"raise_on_click_check\"><property name=\"label\" translatab"
  "le=\"yes\">Raise window when clicking _inside application window</prope"
  "rty><property name=\"use_action_appearance\">False</property><property "
  "name=\"visible\">True</property><property name=\"can_focus\">True</prop"
  "erty><property name=\"receives_default\">False</property><property name"
  "=\"use_underline\">True</property><property name=\"draw_indicator\">Tru"
  "e</property></object></child></object></child><child type=\"label\"><ob"
  "ject class=\"GtkLabel\" id=\"label18\"><property name=\"visible\">True<"
  "/property><property name=\"can_focus\">False</property><property name=\""
  "label\" translatable=\"yes\">&lt;b&gt;Raise on click&lt;/b&gt;</propert"
  "y><property name=\"use_markup\">True</property></object></child></objec"
  "t><packing><property name=\"expand\">False</property><property name=\"f"
  "ill\">False</property><property name=\"position\">3</property></packing"
  "></child></object><packing><property name=\"position\">2</property></pa"
  "cking></child><child type=\"tab\"><object class=\"GtkLabel\" id=\"label"
  "3\"><property name=\"visible\">True</property><property name=\"can_focu"
  "s\">False</property><property name=\"label\" translatable=\"yes\">_Focu"
  "s</property><property name=\"use_underline\">True</property></object><p"
  "acking><property name=\"position\">2</property><property name=\"tab_fil"
  "l\">False</property></packing></child><child><object class=\"GtkBox\" i"
  "d=\"vbox1\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"orientation\">vertical</pr"
  "operty><property name=\"border_width\">12</property><property name=\"sp"
  "acing\">6</property><child><object class=\"GtkFrame\" id=\"frame1\"><pr"
  "operty name=\"visible\">True</property><property name=\"can_focus\">Fal"
  "se</property><property name=\"label_xalign\">0</property><property name"
  "=\"shadow_type\">none</property><child><object class=\"GtkAlignment\" i"
  "d=\"alignment2\"><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">False</property><property name=\"border_width\">6</pro"
  "perty><property name=\"bottom_padding\">6</property><property name=\"le"
  "ft_padding\">12</property><child><object class=\"GtkBox\" id=\"vbox2\">"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "False</property><property name=\"orientation\">vertical</property><prop"
  "erty name=\"spacing\">6</property><child><object class=\"GtkBox\" id=\""
  "hbox8\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"orientation\">horizontal</prop"
  "erty><child><object class=\"GtkCheckButton\" id=\"snap_to_border_check\""
  "><property name=\"label\" translatable=\"yes\">To screen _borders</prop"
  "erty><property name=\"use_action_appearance\">False</property><property"
  " name=\"visible\">True</property><property name=\"can_focus\">True</pro"
  "perty><property name=\"receives_default\">False</property><property nam"
  "e=\"use_underline\">True</property><property name=\"draw_indicator\">Tr"
  "ue</property></object><packing><property name=\"expand\">True</property"
  "><property name=\"fill\">True</property><property name=\"position\">0</"
  "property></packing></child><child><object class=\"GtkCheckButton\" id=\""
  "snap_to_window_check\"><property name=\"label\" translatable=\"yes\">To"
  " other _windows</property><property name=\"use_action_appearance\">Fals"
  "e</property><property name=\"visible\">True</property><property name=\""
  "can_focus\">True</property><property name=\"receives_default\">False</p"
  "roperty><property name=\"use_underline\">True</property><property name="
  "\"draw_indicator\">True</property></object><packing><property name=\"ex"
  "pand\">True</property><property name=\"fill\">True</property><property "
  "name=\"position\">1</property></packing></child></object><packing><prop"
  "erty name=\"expand\">True</property><property name=\"fill\">True</prope"
  "rty><property name=\"position\">0</property></packing></child><child><o"
  "bject class=\"GtkBox\" id=\"vbox5\"><property name=\"visible\">True</pr"
  "operty><property name=\"can_focus\">False</property><property name=\"or"
  "ientation\">vertical</property><property name=\"spacing\">6</property><"
  "child><object class=\"GtkLabel\" id=\"label9\"><property name=\"visible"
  "\">True</property><property name=\"can_focus\">False</property><propert"
  "y name=\"xalign\">0</property><property name=\"label\" translatable=\"y"
  "es\">Dis_tance:</property><property name=\"use_underline\">True</proper"
  "ty><property name=\"mnemonic_widget\">snap_width_scale</property></obje"
  "ct><packing><property name=\"expand\">True</property><property name=\"f"
  "ill\">True</property><property name=\"position\">0</property></packing>"
  "</child><child><object class=\"GtkAlignment\" id=\"alignment6\"><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">False</"
  "property><property name=\"left_padding\">12</property><child><object cl"
  "ass=\"GtkBox\" id=\"hbox1\"><property name=\"visible\">True</property><"
  "property name=\"can_focus\">False</property><property name=\"orientatio"
  "n\">horizontal</property><property name=\"spacing\">12</property><child"
  "><object class=\"GtkLabel\" id=\"label11\"><property name=\"visible\">T"
  "rue</property><property name=\"can_focus\">False</property><property na"
  "me=\"label\" translatable=\"yes\" comments=\"Snapping distance\">&lt;i&"
  "gt;Small&lt;/i&gt;</property><property name=\"use_markup\">True</proper"
  "ty></object><packing><property name=\"expand\">False</property><propert"
  "y name=\"fill\">False</property><property name=\"position\">0</property"
  "></packing></child><child><object class=\"GtkScale\" id=\"snap_width_sc"
  "ale\"><property name=\"visible\">True</property><property name=\"can_fo"
  "cus\">True</property><property name=\"orientation\">horizontal</propert"
  "y><property name=\"adjustment\">adjustment3</property><property name=\""
  "restrict_to_fill_level\">False</property><property name=\"draw_value\">"
  "False</property></object><packing><property name=\"expand\">True</prope"
  "rty><property name=\"fill\">True</property><property name=\"position\">"
  "1</property></packing></child><child><object class=\"GtkLabel\" id=\"la"
  "bel12\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"label\" translatable=\"yes\" c"
  "omments=\"Snapping distance\">&lt;i&gt;Wide&lt;/i&gt;</property><proper"
  "ty name=\"use_markup\">True</property></object><packing><property name="
  "\"expand\">False</property><property name=\"fill\">False</property><pro"
  "perty name=\"position\">2</property></packing></child></object></child>"
  "</object><packing><property name=\"expand\">True</property><property na"
  "me=\"fill\">True</property><property name=\"position\">1</property></pa"
  "cking></child></object><packing><property name=\"expand\">True</propert"
  "y><property name=\"fill\">True</property><property name=\"position\">1<"
  "/property></packing></child></object></child></object></child><child ty"
  "pe=\"label\"><object class=\"GtkLabel\" id=\"label5\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">False</property><"
  "property name=\"label\" translatable=\"yes\">&lt;b&gt;Windows snapping&"
  "lt;/b&gt;</property><property name=\"use_markup\">True</property></obje"
  "ct></child></object><packing><property name=\"expand\">False</property>"
  "<property name=\"fill\">True</property><property name=\"position\">0</p"
  "roperty></packing></child><child><object class=\"GtkFrame\" id=\"frame2"
  "\"><property name=\"visible\">True</property><property name=\"can_focus"
  "\">False</property><property name=\"label_xalign\">0</property><propert"
  "y name=\"shadow_type\">none</property><child><object class=\"GtkAlignme"
  "nt\" id=\"alignment3\"><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">False</property><property name=\"border_width\">"
  "6</property><property name=\"bottom_padding\">6</property><property nam"
  "e=\"left_padding\">12</property><child><object class=\"GtkBox\" id=\"vb"
  "ox4\"><property name=\"visible\">True</property><property name=\"can_fo"
  "cus\">False</property><property name=\"orientation\">vertical</property"
  "><property name=\"spacing\">6</property><child><object class=\"GtkBox\""
  " id=\"hbox9\"><property name=\"visible\">True</property><property name="
  "\"can_focus\">False</property><property name=\"orientation\">horizontal"
  "</property><child><object class=\"GtkCheckButton\" id=\"wrap_workspaces"
  "_check\"><property name=\"label\" translatable=\"yes\">With the mouse _"
  "pointer</property><property name=\"use_action_appearance\">False</prope"
  "rty><property name=\"visible\">True</property><property name=\"can_focu"
  "s\">True</property><property name=\"receives_default\">False</property>"
  "<property name=\"use_underline\">True</property><property name=\"draw_i"
  "ndicator\">True</property></object><packing><property name=\"expand\">T"
  "rue</property><property name=\"fill\">True</property><property name=\"p"
  "osition\">0</property></packing></child><child><object class=\"GtkCheck"
  "Button\" id=\"wrap_windows_check\"><property name=\"label\" translatabl"
  "e=\"yes\">With a _dragged window</property><property name=\"use_action_"
  "appearance\">False</property><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">True</property><property name=\"receives_d"
  "efault\">False</property><property name=\"use_underline\">True</propert"
  "y><property name=\"draw_indicator\">True</property></object><packing><p"
  "roperty name=\"expand\">True</property><property name=\"fill\">True</pr"
  "operty><property name=\"position\">1</property></packing></child></obje"
  "ct><packing><property name=\"expand\">True</property><property name=\"f"
  "ill\">True</property><property name=\"position\">0</property></packing>"
  "</child><child><object class=\"GtkBox\" id=\"vbox6\"><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">False</property><p"
  "roperty name=\"orientation\">vertical</property><property name=\"spacin"
  "g\">6</property><child><object class=\"GtkLabel\" id=\"label10\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">False<"
  "/property><property name=\"xalign\">0</property><property name=\"label\""
  " translatable=\"yes\">_Edge resistance:</property><property name=\"use_"
  "underline\">True</property><property name=\"selectable\">True</property"
  "><property name=\"mnemonic_widget\">wrap_resistance_scale</property></o"
  "bject><packing><property name=\"expand\">False</property><property name"
  "=\"fill\">False</property><property name=\"position\">0</property></pac"
  "king></child><child><object class=\"GtkAlignment\" id=\"alignment4\"><p"
  "roperty name=\"visible\">True</property><property name=\"can_focus\">Fa"
  "lse</property><property name=\"left_padding\">12</property><child><obje"
  "ct class=\"GtkBox\" id=\"hbox2\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">False</property><property name=\"orien"
  "tation\">horizontal</property><property name=\"spacing\">12</property><"
  "child><object class=\"GtkLabel\" id=\"label13\"><property name=\"visibl"
  "e\">True</property><property name=\"can_focus\">False</property><proper"
  "ty name=\"label\" translatable=\"yes\" comments=\"Edge resistance\">&lt"
  ";i&gt;Small&lt;/i&gt;</property><property name=\"use_markup\">True</pro"
  "perty></object><packing><property name=\"expand\">False</property><prop"
  "erty name=\"fill\">False</property><property name=\"position\">0</prope"
  "rty></packing></child><child><object class=\"GtkScale\" id=\"wrap_resis"
  "tance_scale\"><property name=\"visible\">True</property><property name="
  "\"can_focus\">True</property><property name=\"orientation\">horizontal<"
  "/property><property name=\"adjustment\">adjustment4</property><property"
  " name=\"restrict_to_fill_level\">False</property><property name=\"draw_"
  "value\">False</property></object><packing><property name=\"expand\">Tru"
  "e</property><property name=\"fill\">True</property><property name=\"pos"
  "ition\">1</property></packing></child><child><object class=\"GtkLabel\""
  " id=\"label14\"><property name=\"visible\">True</property><property nam"
  "e=\"can_focus\">False</property><property name=\"label\" translatable=\""
  "yes\" comments=\"Edge resistance\">&lt;i&gt;Wide&lt;/i&gt;</property><p"
  "roperty name=\"use_markup\">True</property></object><packing><property "
  "name=\"expand\">False</property><property name=\"fill\">False</property"
  "><property name=\"position\">2</property></packing></child></object></c"
  "hild></object><packing><property name=\"expand\">True</property><proper"
  "ty name=\"fill\">True</property><property name=\"position\">1</property"
  "></packing></child></object><packing><property name=\"expand\">True</pr"
  "operty><property name=\"fill\">True</property><property name=\"position"
  "\">1</property></packing></child></object></child></object></child><chi"
  "ld type=\"label\"><object class=\"GtkLabel\" id=\"label6\"><property na"
  "me=\"visible\">True</property><property name=\"can_focus\">False</prope"
  "rty><property name=\"label\" translatable=\"yes\">&lt;b&gt;Wrap workspa"
  "ces when reaching the screen edge&lt;/b&gt;</property><property name=\""
  "use_markup\">True</property></object></child></object><packing><propert"
  "y name=\"expand\">False</property><property name=\"fill\">True</propert"
  "y><property name=\"position\">1</property></packing></child><child><obj"
  "ect class=\"GtkFrame\" id=\"frame3\"><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">False</property><property name=\"l"
  "abel_xalign\">0</property><property name=\"shadow_type\">none</property"
  "><child><object class=\"GtkAlignment\" id=\"alignment5\"><property name"
  "=\"visible\">True</property><property name=\"can_focus\">False</propert"
  "y><property name=\"border_width\">6</property><property name=\"bottom_p"
  "adding\">6</property><property name=\"left_padding\">12</property><chil"
  "d><object class=\"GtkBox\" id=\"hbox6\"><property name=\"visible\">True"
  "</property><property name=\"can_focus\">False</property><property name="
  "\"orientation\">horizontal</property><child><object class=\"GtkCheckBut"
  "ton\" id=\"box_move_check\"><property name=\"label\" translatable=\"yes"
  "\">When _moving</property><property name=\"use_action_appearance\">Fals"
  "e</property><property name=\"visible\">True</property><property name=\""
  "can_focus\">True</property><property name=\"receives_default\">False</p"
  "roperty><property name=\"use_underline\">True</property><property name="
  "\"draw_indicator\">True</property></object><packing><property name=\"ex"
  "pand\">True</property><property name=\"fill\">True</property><property "
  "name=\"position\">0</property></packing></child><child><object class=\""
  "GtkCheckButton\" id=\"box_resize_check\"><property name=\"label\" trans"
  "latable=\"yes\">When _resizing</property><property name=\"use_action_ap"
  "pearance\">False</property><property name=\"visible\">True</property><p"
  "roperty name=\"can_focus\">True</property><property name=\"receives_def"
  "ault\">False</property><property name=\"use_underline\">True</property>"
  "<property name=\"draw_indicator\">True</property></object><packing><pro"
  "perty name=\"expand\">True</property><property name=\"fill\">True</prop"
  "erty><property name=\"position\">1</property></packing></child></object"
  "></child></object></child><child type=\"label\"><object class=\"GtkLabe"
  "l\" id=\"label7\"><property name=\"visible\">True</property><property n"
  "ame=\"can_focus\">False</property><property name=\"label\" translatable"
  "=\"yes\">&lt;b&gt;Hide content of windows&lt;/b&gt;</property><property"
  " name=\"use_markup\">True</property></object></child></object><packing>"
  "<property name=\"expand\">False</property><property name=\"fill\">True<"
  "/property><property name=\"position\">2</property></packing></child><ch"
  "ild><object class=\"GtkFrame\" id=\"frame4\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property><property "
  "name=\"label_xalign\">0</property><property name=\"shadow_type\">none</"
  "property><child><object class=\"GtkAlignment\" id=\"alignment13\"><prop"
  "erty name=\"visible\">True</property><property name=\"can_focus\">False"
  "</property><property name=\"top_padding\">6</property><property name=\""
  "bottom_padding\">6</property><property name=\"left_padding\">18</proper"
  "ty><child><object class=\"GtkComboBox\" id=\"double_click_action_combo\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"tooltip_text\" translatable=\"yes\">T"
  "he action to perform when the title-bar is double-clicked</property></o"
  "bject></child></object></child><child type=\"label\"><object class=\"Gt"
  "kLabel\" id=\"label8\"><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">False</property><property name=\"label\" transla"
  "table=\"yes\">&lt;b&gt;Double click _action&lt;/b&gt;</property><proper"
  "ty name=\"use_markup\">True</property><property name=\"use_underline\">"
  "True</property><property name=\"mnemonic_widget\">double_click_action_c"
  "ombo</property></object></child></object><packing><property name=\"expa"
  "nd\">False</property><property name=\"fill\">True</property><property n"
  "ame=\"position\">3</property></packing></child></object><packing><prope"
  "rty name=\"position\">3</property></packing></child><child type=\"tab\""
  "><object class=\"GtkLabel\" id=\"label4\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"label\" translatable=\"yes\">Ad_vanced</property><property name=\"u"
  "se_underline\">True</property></object><packing><property name=\"positi"
  "on\">3</property><property name=\"tab_fill\">False</property></packing>"
  "</child></object><packing><property name=\"expand\">True</property><pro"
  "perty name=\"fill\">True</property><property name=\"position\">1</prope"
  "rty></packing></child></object></child><action-widgets><action-widget r"
  "esponse=\"-11\">help_button</action-widget><action-widget response=\"0\""
  ">close_button</action-widget></action-widgets></object><object class=\""
  "GtkSizeGroup\" id=\"sizegroup1\"><property name=\"mode\">vertical</prop"
  "erty><widgets><widget name=\"alignment17\"/><widget name=\"alignment18\""
  "/></widgets></object></interface>"
};

static const unsigned xfwm4_dialog_ui_length = 50461u;

